/* 

   libj - conio_out.c            Copyright (C) 2001-1998 Justin David Smith
   justins@chaos2.org            http://chaos2.org/
    
   Console output


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#include <libjconio.h>
#include <sys/ioctl.h>
#include <setjmp.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <termcap.h>
#include <termios.h>
#include <unistd.h>


#ifndef sigmask
#define sigmask(s)               (1 << ((s) - 1))
#endif

#ifndef OXTABS
#define OXTABS XTABS
#endif


#define _io_putpad(s)            tputs((s), 1, _io_put)
#define _io_putstr(s)            fputs((s), stdout)
#define _io_moveto(r, c)         _io_putpad(tgoto(CMstr, (c), (r)))


int _io_put __P((int));


static int __io_isset;           /* If nonzero, the terminal is set */
static struct termios __io_oldtty;/*Original terminal settings */
static struct termios __io_newtty;/*New terminal settings */
static void (*__io_tstp)(int);
static void (*__io_ttou)(int);

static void io_scr_stop(int);
static void io_stopset(int);

/* Capabilities from TERMCAP */
char PC;       /* Globals required by tgoto */
char *BC;
char *UP;     
#ifndef __linux__
speed_t ospeed;
#endif
  

static char *bcstr;     /* backspace char */
static char *CEstr;     /* Clear to end of line */
static char *CLstr;     /* clear screen */
static char *CMstr;     /* Cursor motion string */
static char *CRstr;     /* \r equivalent string */
static char *HOstr;     /* Home string */
static char *LLstr;     /* last line first column */
static char *pcstr;     /* padding char */
static char *TEstr;     /* end cursor motion mode */
static char *TIstr;     /* begin cursor motion mode */
static char *SEstr;     /* end standout mode */
static char *SOstr;     /* begin standout mode */


static int COnum;       /* co# value */
static int LInum;       /* li# value */
static int bsflag;
static int xsflag;
static int ncflag;
static int sgnum;
static int MSflag;      /* Can move in standout */


static int Rows;        /* number of rows */
static int Cols;        /* number of columns */


typedef struct _tcinfo {
   char name[3];
   void *data;
} tcinfo;


typedef struct _tcsinfo {
   char name[3];
   char **data;
} tcsinfo;


typedef struct _tcninfo {
   char name[3];
   int *data;
} tcninfo;


static tcsinfo tcstrings[] = 
{  {  "bc", &bcstr   },
   {  "ce", &CEstr   },
   {  "cl", &CLstr   },
   {  "cm", &CMstr   },
   {  "cr", &CRstr   },
   {  "le", &BC      },
   {  "pc", &pcstr   },
   {  "se", &SEstr   },
   {  "so", &SOstr   },
   {  "te", &TEstr   },
   {  "ti", &TIstr   },
   {  "up", &UP      },
   {  "ho", &HOstr   },
   {  "",   NULL     } 
};

static tcninfo tcflags[] = 
{  {  "bs",  &bsflag },
   {  "ms",  &MSflag },
   {  "nc",  &ncflag },
   {  "xs",  &xsflag },
   {  "",    NULL    } 
};

static tcninfo tcnums[] =
{  {  "co",  &COnum  },
   {  "li",  &LInum  },
   {  "sg",  &sgnum  },
   {  "",    NULL    } 
};


static char combuf[0x1000];
static char tbuf[0x1000];


/* Character output */

int _io_put(int c) {
/* internal-io-put-character

   This needs to be a function (not inlined), to appease tputs. */

   return(putchar(c));
   
}


int io_putchar(char c) {
/* io-put-character */

   return(putchar(c));
   
}


int io_putstr(char *s) {
/* io-put-string */

   return(_io_putstr(s));
   
}


int io_putnstr(char *s, unsigned int size) {
/* io-put-string-size */

   char *p;
   char c;
   int rtn;
   
   p = s;
   while(*p && size) {
      p++;
      size--;
   }
   if(*p) {
      c = *p;
      *p = 0;
   } else c = 0;
   
   rtn = _io_putstr(s);
   if(c) *p = c;
   return(rtn);

}


int io_scr_init() {
/* io-Screen-Init */

   char *term;
   char *fill;
   
   tcinfo *p;
      
   if(!(term = getenv("TERM"))) {
      fprintf(stderr, "TERM variable unset\n");
      return(-1);
   }
   
   if(tgetent(tbuf, term) <= 0) {
      fprintf(stderr, "cannot find termcap for '%s'\n", term);
      return(-1);
   }
   fill = combuf;
   
   for(p = (tcinfo *)tcstrings; p->data; p++) 
      *(char **)p->data = tgetstr(p->name, &fill);
   for(p = (tcinfo *)tcflags; p->data; p++) 
      *(int *)p->data = tgetflag(p->name);
   for(p = (tcinfo *)tcnums; p->data; p++) 
      *(int *)p->data = tgetnum(p->name);
      
   if(bsflag) BC = "\b";
   else if(!BC && bcstr) BC = bcstr;
   
   if(!CLstr) {
      fprintf(stderr, "cannot clear screen for '%s'\n", term);
      return(-1);
   }
   if(!CMstr || !UP || !BC) {
      fprintf(stderr, "cannot position cursor for '%s'\n", term);
      return(-1);
   }

   PC = pcstr ? *pcstr : 0;
   if(sgnum >= 0 || xsflag) SOstr = SEstr = NULL;
   if(ncflag) CRstr = NULL;
   else if(!CRstr) CRstr = "\r";
   return(0);
   
}


static jmp_buf __io_scr_onstop;


static void io_stopset(int sig) {
/* io-Stop-Set */

   sigset_t sigset;
   
   signal(sig, SIG_DFL);
   kill(getpid(), sig);
   sigemptyset(&sigset);
   sigaddset(&sigset, sig);
   sigprocmask(SIG_UNBLOCK, &sigset, (sigset_t *)0);
   longjmp(__io_scr_onstop, 1);
   
}


static void io_scr_stop(int sig) {
/* io-Screen-Stop */

   sigset_t sigset;
   
   io_scr_end();
   kill(getpid(), sig);
   sigemptyset(&sigset);
   sigaddset(&sigset, sig);
   sigprocmask(SIG_UNBLOCK, &sigset, (sigset_t *)0);
   io_scr_set();

}


int io_scr_set() {
/* io-screen-set */

   struct winsize ws;
   sigset_t sigset;
   sigset_t osigset;
            
   sigemptyset(&sigset);
   sigaddset(&sigset, SIGTSTP);
   sigaddset(&sigset, SIGTTOU);
   sigprocmask(SIG_BLOCK, &sigset, &osigset);
   if((__io_tstp = signal(SIGTSTP, io_stopset)) == SIG_IGN) signal(SIGTSTP, SIG_IGN);
   if((__io_ttou = signal(SIGTTOU, io_stopset)) == SIG_IGN) signal(SIGTTOU, SIG_IGN);

   /* Changing terminal settings */
   setjmp(__io_scr_onstop);
   sigprocmask(SIG_SETMASK, &osigset, (sigset_t *)0);
   Rows = 0;
   Cols = 0;
   if(ioctl(0, TIOCGWINSZ, &ws) == 0) {
      Rows = ws.ws_row;
      Cols = ws.ws_col;
   }
   if(Rows == 0) Rows = LInum;
   if(Cols == 0) Cols = COnum;

   if(tcgetattr(0, &__io_oldtty) < 0) {
      fprintf(stderr, "tcgetattr() fails");
      return(-1);
   }
   __io_newtty = __io_oldtty;
   __io_newtty.c_lflag &= ~(ICANON | ECHO);
   __io_newtty.c_oflag &= ~OXTABS;
   if (tcsetattr(0, TCSADRAIN, &__io_newtty) < 0) {
      fprintf(stderr, "tcsetattr() fails");
      return(-1);
   }
   ospeed = cfgetospeed(&__io_newtty);
   sigprocmask(SIG_BLOCK, &sigset, &osigset);
                                                                              
   /* Done changing settings */
   if(TIstr) _io_putstr(TIstr); /* termcap(5) says this is not padded */
   if(__io_tstp != SIG_IGN) signal(SIGTSTP, io_scr_stop);
   if(__io_ttou != SIG_IGN) signal(SIGTTOU, __io_ttou);
   __io_isset = 1;
   sigprocmask(SIG_SETMASK, &osigset, (sigset_t *)0);
   io_scr_clear();
                                    
   return(0);
      
}


int io_scr_end() {
/* io-screen-end */

   sigset_t sigset, osigset;

   sigemptyset(&sigset);
   sigaddset(&sigset, SIGTSTP);
   sigaddset(&sigset, SIGTTOU);
   sigprocmask(SIG_BLOCK, &sigset, &osigset);
   
   /* move cursor to last line */
   if(LLstr) _io_putstr(LLstr); /* termcap(5) says this is not padded */
   else _io_moveto(Rows - 1, 0);

   /* exit screen mode */
   if(TEstr) _io_putstr(TEstr); /* termcap(5) says this is not padded */
   fflush(stdout);
   tcsetattr(0, TCSADRAIN, &__io_oldtty);
   __io_isset = 0;
   
   /* restore signals */
   signal(SIGTSTP, __io_tstp);
   sigprocmask(SIG_SETMASK, &osigset, (sigset_t *)0);
   return(0);

}


int io_in_scr() {
/* io-in-screen? */

   return(__io_isset);

}


int io_move(int row, int col) {
/* io-move */

   _io_moveto(row, col);
   return(0);
   
}


int io_scr_clear() {
/* io-screen-clear */

   _io_putpad(CLstr);
   return(0);
         
}


int io_scr_clear_eol() {
/* io-screen-clear-to-end-of-line */

   _io_putpad(CEstr);
   return(0);

}


int io_scr_clear_line(int row) {

   _io_moveto(row, 0);
   _io_putpad(CEstr);
   return(0);
   
}


int io_standoff(int f) {

   if(f) _io_putpad(SOstr);
   else _io_putpad(SEstr);
   return(0);

}


int io_first() {

   _io_putpad(HOstr);
   return(0);

}


int io_last() {

   _io_putpad(LLstr);
   return(0);

}


int io_flush() {

   fflush(stdout);
   return(0);
   
}

