/* 

   libj - libjconio.h            Copyright (C) 2001-1998 Justin David Smith
   justins@chaos2.org            http://chaos2.org/
    
   Header file for console I/O library


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#ifndef __libj_conio_included
#define __libj_conio_included


/* Include files */
#include <libj.h>
#include <stdlib.h>
#include <stdio.h>
#if TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
#else
#  if HAVE_SYS_TIME_H
#    include <sys/time.h>
#  else
#    include <time.h>
#  endif
#endif
#include <unistd.h>


/* Character input */

int io_char_avail();
int io_read_char();
int io_wait_char(struct timeval *delay);
int io_read_string(char *dest, unsigned int size);
int io_wait_string(char *dest, unsigned int size, struct timeval *delay);
int io_eat();
int io_idle(struct timeval *delay);
int io_sleep(struct timeval *delay);
long io_read_key();
long io_wait_key(struct timeval *delay);
void io_lookup_key(long key, char **name, long *basekey, long *modifier);


/* Character output */

int io_putchar(char c);
int io_putstr(char *s);
int io_putnstr(char *s, unsigned int size);
int io_scr_init();
int io_scr_set();
int io_scr_end();
int io_in_scr();
int io_move(int row, int col);
int io_scr_clear();
int io_scr_clear_eol();
int io_scr_clear_line(int row);
int io_standoff(int f);
int io_first();
int io_last();
int io_flush();


/* Miscellaneous */

unsigned int libjconio_version();





/* ANSI output escape sequences */

/* General escapes */
#define C_ANSI_CLEAR             "\e[2J"
#define C_ANSI_CLRSCR            "\e[H\e[2J"
#define C_ANSI_RESET_CURSOR      "\e[r"
#define C_ANSI_SET_CURSOR        "\e[%d;%dH"
#define C_ANSI_RESET             "\e[0;37;40m"
#define C_ANSI_RESET_END         "\e[0m"
#define C_ANSI_UNDERLINE         "\e[4m"
#define C_ANSI_BOLD              "\e[1m"

/* Foreground colors */
#define C_ANSI_FG_BLACK          "\e[30m"
#define C_ANSI_FG_GREEN          "\e[32m"
#define C_ANSI_FG_RED            "\e[31m"
#define C_ANSI_FG_YELLOW         "\e[33m"
#define C_ANSI_FG_BLUE           "\e[34m"
#define C_ANSI_FG_VIOLET         "\e[35m"
#define C_ANSI_FG_CYAN           "\e[36m"
#define C_ANSI_FG_WHITE          "\e[37m"

/* Background colors */
#define C_ANSI_BG_BLACK          "\e[40m"
#define C_ANSI_BG_GREEN          "\e[42m"
#define C_ANSI_BG_RED            "\e[41m"
#define C_ANSI_BG_YELLOW         "\e[43m"
#define C_ANSI_BG_BLUE           "\e[44m"
#define C_ANSI_BG_VIOLET         "\e[45m"
#define C_ANSI_BG_CYAN           "\e[46m"
#define C_ANSI_BG_WHITE          "\e[47m"

/* Combinations */
#define C_ANSI_BLACK             "\e[0;30m"
#define C_ANSI_GRAY              "\e[0;37m"
#define C_ANSI_LIGHT_GRAY        C_ANSI_GRAY
#define C_ANSI_DARK_GRAY         "\e[1;30m"
#define C_ANSI_BOLD_BLACK        C_ANSI_DARK_GRAY
#define C_ANSI_WHITE             "\e[1;37m"
#define C_ANSI_BOLD_WHITE        C_ANSI_WHITE
#define C_ANSI_BOLD_GREEN        "\e[1;32m"
#define C_ANSI_BOLD_RED          "\e[1;31m"
#define C_ANSI_BOLD_YELLOW       "\e[1;33m"
#define C_ANSI_BOLD_BLUE         "\e[1;34m"
#define C_ANSI_BOLD_VIOLET       "\e[1;35m"
#define C_ANSI_BOLD_CYAN         "\e[1;36m"
#define C_ANSI_GREEN             "\e[0;32m"
#define C_ANSI_RED               "\e[0;31m"
#define C_ANSI_YELLOW            "\e[0;33m"
#define C_ANSI_BLUE              "\e[0;34m"
#define C_ANSI_VIOLET            "\e[0;35m"
#define C_ANSI_CYAN              "\e[0;36m"

#define ANSI_CLEAR               printf(C_ANSI_CLEAR)
#define ANSI_CLRSCR              printf(C_ANSI_CLRSCR)
#define ANSI_RESET_CURSOR        printf(C_ANSI_RESET_CURSOR)
#define ANSI_SET_CURSOR(r, c)    printf(C_ANSI_SET_CURSOR, (r), (c))
#define ANSI_RESET               printf(C_ANSI_RESET)
#define ANSI_RESET_END           printf(C_ANSI_RESET_END)
#define ANSI_UNDERLINE           printf(C_ANSI_UNDERLINE)
#define ANSI_BOLD                printf(C_ANSI_BOLD)
#define ANSI_GREEN               printf(C_ANSI_FG_GREEN)
#define ANSI_RED                 printf(C_ANSI_FG_RED)
#define ANSI_YELLOW              printf(C_ANSI_FG_YELLOW)
#define ANSI_BLUE                printf(C_ANSI_FG_BLUE)
#define ANSI_VIOLET              printf(C_ANSI_FG_VIOLET)
#define ANSI_CYAN                printf(C_ANSI_FG_CYAN)
#define ANSI_GRAY                printf(C_ANSI_FG_GRAY)





/* Key definitions */
#define  IO_MOD_ALT     1
#define  IO_MOD_SHIFT   2
#define  IO_MOD_CTRL    4


/* General characters */

#define  IO_KEY_ESC     0x1b
#define  IO_KEY_TAB     0x09
#define  IO_KEY_BKSP    0x7f
#define  IO_KEY_RETURN  0x0a
#define  IO_KEY_PAUSE   0x1b5b50
#define  IO_KEY_SPACE   0x20

#define  IO_KEY_ESC_A   0x1b1b
#define  IO_KEY_TAB_A   0x1b09
#define  IO_KEY_BKSP_A  0x1b7f
#define  IO_KEY_RETURN_A 0x1b0a
#define  IO_KEY_SPACE_A 0x1b20

#define  IO_KEY_BKSP_C  0x08


/* Function keys */

#define  IO_KEY_F1      0x1b5b5b41
#define  IO_KEY_F2      0x1b5b5b42
#define  IO_KEY_F3      0x1b5b5b43
#define  IO_KEY_F4      0x1b5b5b44
#define  IO_KEY_F5      0x1b5b5b45
#define  IO_KEY_F6      0x1b5b3137/*7e*/
#define  IO_KEY_F7      0x1b5b3138/*7e*/
#define  IO_KEY_F8      0x1b5b3139/*7e*/
#define  IO_KEY_F9      0x1b5b3230/*7e*/
#define  IO_KEY_F10     0x1b5b3231/*7e*/
#define  IO_KEY_F11     0x1b5b3233/*7e*/
#define  IO_KEY_F12     0x1b5b3234/*7e*/

#define  IO_KEY_F1_S    0x1b5b3233/*7e*/
#define  IO_KEY_F2_S    0x1b5b3234/*7e*/
#define  IO_KEY_F3_S    0x1b5b3235/*7e*/
#define  IO_KEY_F4_S    0x1b5b3236/*7e*/
#define  IO_KEY_F5_S    0x1b5b3238/*7e*/
#define  IO_KEY_F6_S    0x1b5b3239/*7e*/
#define  IO_KEY_F7_S    0x1b5b3331/*7e*/
#define  IO_KEY_F8_S    0x1b5b3332/*7e*/
#define  IO_KEY_F9_S    0x1b5b3333/*7e*/
#define  IO_KEY_F10_S   0x1b5b3334/*7e*/
#define  IO_KEY_F11_S   0x1b5b3233/*7e*/
#define  IO_KEY_F12_S   0x1b5b3234/*7e*/

#define  IO_KEY_F1_C    0x1b5b5b41
#define  IO_KEY_F2_C    0x1b5b5b42
#define  IO_KEY_F3_C    0x1b5b5b43
#define  IO_KEY_F4_C    0x1b5b5b44
#define  IO_KEY_F5_C    0x1b5b5b45
#define  IO_KEY_F6_C    0x1b5b3137/*7e*/
#define  IO_KEY_F7_C    0x1b5b3138/*7e*/
#define  IO_KEY_F8_C    0x1b5b3139/*7e*/
#define  IO_KEY_F9_C    0x1b5b3230/*7e*/
#define  IO_KEY_F10_C   0x1b5b3231/*7e*/
#define  IO_KEY_F11_C   0x1b5b3233/*7e*/
#define  IO_KEY_F12_C   0x1b5b3234/*7e*/


/* Cursor movement */

#define  IO_KEY_HOME    0x1b5b317e
#define  IO_KEY_INS     0x1b5b327e
#define  IO_KEY_DEL     0x1b5b337e
#define  IO_KEY_END     0x1b5b347e
#define  IO_KEY_PGUP    0x1b5b357e
#define  IO_KEY_PGDN    0x1b5b367e
                                                                                 
#define  IO_KEY_UP      0x1b5b41
#define  IO_KEY_DOWN    0x1b5b42
#define  IO_KEY_RIGHT   0x1b5b43
#define  IO_KEY_LEFT    0x1b5b44
#define  IO_KEY_CENTER  0x1b5b47


/* Alphanumeric keyboard */

#define  IO_KEY_A_C     0x01
#define  IO_KEY_B_C     0x02
#define  IO_KEY_C_C     0x03
#define  IO_KEY_D_C     0x04
#define  IO_KEY_E_C     0x05
#define  IO_KEY_F_C     0x06
#define  IO_KEY_G_C     0x07
#define  IO_KEY_H_C     0x08
#define  IO_KEY_I_C     0x09
#define  IO_KEY_J_C     0x0a
#define  IO_KEY_K_C     0x0b
#define  IO_KEY_L_C     0x0c
#define  IO_KEY_M_C     0x0d
#define  IO_KEY_N_C     0x0e
#define  IO_KEY_O_C     0x0f
#define  IO_KEY_P_C     0x10
#define  IO_KEY_Q_C     0x11
#define  IO_KEY_R_C     0x12
#define  IO_KEY_S_C     0x13
#define  IO_KEY_T_C     0x14
#define  IO_KEY_U_C     0x15
#define  IO_KEY_V_C     0x16
#define  IO_KEY_W_C     0x17
#define  IO_KEY_X_C     0x18
#define  IO_KEY_Y_C     0x19
#define  IO_KEY_Z_C     0x1a
#define  IO_KEY_3_C     0x1b
#define  IO_KEY_4_C     0x1c
#define  IO_KEY_5_C     0x1d
#define  IO_KEY_6_C     0x1e
#define  IO_KEY_7_C     0x1f

#define  IO_KEY_TILDE_A 0x1b60
#define  IO_KEY_MINUS_A 0x1b2d
#define  IO_KEY_PLUS_A  0x1b3d
#define  IO_KEY_OBRAC_A 0x1b5b
#define  IO_KEY_CBRAC_A 0x1b5d
#define  IO_KEY_COLON_A 0x1b3b
#define  IO_KEY_QUOTE_A 0x1b27
#define  IO_KEY_COMMA_A 0x1b2c
#define  IO_KEY_DOT_A   0x1b2e
#define  IO_KEY_SLASH_A 0x1b2f
#define  IO_KEY_PIPE_A  0x1b5c
#define  IO_KEY_STAR_A  0x1b2a

#define  IO_KEY_0_A     0x1b30
#define  IO_KEY_1_A     0x1b31
#define  IO_KEY_2_A     0x1b32
#define  IO_KEY_3_A     0x1b33
#define  IO_KEY_4_A     0x1b34
#define  IO_KEY_5_A     0x1b35
#define  IO_KEY_6_A     0x1b36
#define  IO_KEY_7_A     0x1b37
#define  IO_KEY_8_A     0x1b38
#define  IO_KEY_9_A     0x1b39
#define  IO_KEY_A_A     0x1b61
#define  IO_KEY_B_A     0x1b62
#define  IO_KEY_C_A     0x1b63
#define  IO_KEY_D_A     0x1b64
#define  IO_KEY_E_A     0x1b65
#define  IO_KEY_F_A     0x1b66
#define  IO_KEY_G_A     0x1b67
#define  IO_KEY_H_A     0x1b68
#define  IO_KEY_I_A     0x1b69
#define  IO_KEY_J_A     0x1b6a
#define  IO_KEY_K_A     0x1b6b
#define  IO_KEY_L_A     0x1b6c
#define  IO_KEY_M_A     0x1b6d
#define  IO_KEY_N_A     0x1b6e
#define  IO_KEY_O_A     0x1b6f
#define  IO_KEY_P_A     0x1b70
#define  IO_KEY_Q_A     0x1b71
#define  IO_KEY_R_A     0x1b72
#define  IO_KEY_S_A     0x1b73
#define  IO_KEY_T_A     0x1b74
#define  IO_KEY_U_A     0x1b75
#define  IO_KEY_V_A     0x1b76
#define  IO_KEY_W_A     0x1b77
#define  IO_KEY_X_A     0x1b78
#define  IO_KEY_Y_A     0x1b79
#define  IO_KEY_Z_A     0x1b7a




#endif

