/* 

   libj - _file_macro.h          Copyright (C) 2001-1998 Justin David Smith
   justins@chaos2.org            http://chaos2.org/
    
   Internal Macros for File I/O library


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#ifndef __libj__file_macro_included
#define __libj__file_macro_included


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>


/* Status macros */

#define __file_is_file(f)        ((f)->f != -1)
#define __file_is_memory(f)      ((f)->f == -1 && (f)->buf)
#define __file_is_null(f)        (!__file_is_file(f) && !__file_is_memory(f))
#define __file_is_readable(f)    ((((f)->mode & O_ACCMODE) == O_RDONLY) || \
                                    (((f)->mode & O_ACCMODE) == O_RDWR))
#define __file_is_writable(f)    ((((f)->mode & O_ACCMODE) == O_WRONLY) || \
                                    (((f)->mode & O_ACCMODE) == O_RDWR))


/* Mode translation */

#define __file_mode_from_char(m) ((m) == 'w' || (m) == 'W' ? O_WRONLY | O_CREAT | O_TRUNC : \
                                    (m) == 'a' || (m) == 'A' ? O_WRONLY | O_CREAT | O_APPEND : \
                                    O_RDONLY)


/* Buffer manip */

#define __file_release_buffer(f) if((f)->release && (f)->buf) free((f)->buf); \
                                    (f)->buf = (f)->bufptr = NULL; \
                                    (f)->bufsize = 0, (f)->release = false
#define __file_buf_past_end(f)   ((f)->bufptr >= (f)->buf + (f)->bufsize)
#define __file_buf_at_begin(f)   ((f)->bufptr == (f)->buf)
#define __file_buf_set_begin(f)  (f)->bufptr = (f)->buf


/* Read/Wr buffer */

int   __file_read_buffer(file *f);
bool  __file_write_buffer(file *f);


/* Line column macros */

#define __file_newline_char(ch)  ((ch) == '\0' || (ch) == '\n')
#define __file_next_line(f)      (f)->line++, (f)->col = 1
#define __file_next_column(f)    (f)->col++
#define __file_next_byte(f)      (f)->byte++;
#define __file_advance_line_column(f,ch)  __file_next_byte(f); \
                                    if(__file_newline_char(ch)) __file_next_line(f); \
                                    else __file_next_column(f)


#endif
