/* 

   libj - file.c                 Copyright (C) 2001-1998 Justin David Smith
   justins@chaos2.org            http://chaos2.org/
    
   Main functions for file I/O library


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#include <libjfile.h>
#include <_file_macro.h>



bool file_is_file(const file *f) {
/* Is-File
   Returns true if the file pointer specified is actually a file;
   returns false if the file pointer specified is a "fake file". */
   
   if(f != NULL) return(__file_is_file(f));
   return(false);

}



bool file_is_memory(const file *f) {
/* Is-Memory
   Opposite of isFile, except this also returns false if <f> == NULL. */

   if(f != NULL) return(__file_is_memory(f));
   return(false);
   
}



bool file_is_null(const file *f) {
/* Is-Null */
   
   if(f != NULL) return(__file_is_null(f));
   return(false);
   
}



bool file_is_readable(const file *f) {
/* Is-Readable */

   if(f != NULL) return(__file_is_readable(f));
   return(false);

}


bool file_is_writable(const file *f) {
/* Is-Writable */

   if(f != NULL) return(__file_is_writable(f));
   return(false);

}


const char *file_name(const file *f) {
/* File-Name */

   if(f != NULL) return(f->name);
   return(NULL);
   
}



sizea file_line(const file *f) {
/* File-Line */

   if(f != NULL) return(f->line);
   return(0);
   
}



void file_set_line(file *f, sizea line) {
/* File-Set-Line */

   if(f != NULL) f->line = line;
   
}



sizea file_column(const file *f) {
/* File-Column */

   if(f != NULL) return(f->col);
   return(0);
   
}



void file_set_column(file *f, sizea col) {
/* File-Set-Column */

   if(f != NULL) f->col = col;
   
}



void file_set_line_column(file *f, sizea line, sizea col) {
/* File-Set-Line-Column */

   if(f != NULL) {
      f->line = line;
      f->col = col;
   }
   
}



sizea file_byte(const file *f) {
/* File-Byte */

   if(f != NULL) return(f->byte);
   return(0);

}



void file_set_byte(file *f, sizea b) {
/* File-Set-Byte */

   if(f != NULL) f->byte = b;
   
}



bool file_eof(const file *f) {
/* File-EOF */

   if(f != NULL) {
      if(__file_is_null(f)) return(false);
      if(__file_is_memory(f)) return(f->bufptr >= f->buf + f->bufsize);
      return(f->eof && f->bufptr >= f->buf + f->bufsize);
   }
   return(true);
   
}



bool f_flush(file *f) {
/* File-Flush */

   return(__file_write_buffer(f));

}


