/* 

   libj - file_read.c            Copyright (C) 2001-1998 Justin David Smith    
   justins@chaos2.org            http://chaos2.org/
    
   File input


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#include <libjfile.h>
#include <_file_macro.h>



int __file_read_buffer(file *f) {
/* Internal-File-Read-Buffer */

   int num;

   if(__file_is_file(f) && __file_is_readable(f) && __file_buf_past_end(f)) { 
      num = -1;
      if(f->func) {
         if(f->hooks.fread != NULL) num = f->hooks.fread((f), (f)->f, (f)->buf, (f)->bufsize);
      } else num = read((f)->f, (f)->buf, (f)->bufsize); 
      if(num != -1) {
         __file_buf_set_begin(f);
         if(num < (f)->bufsize) {
            (f)->eof = true;
            (f)->bufsize = num;
         }
         return(num);
      }
   }
   return(0);

}
                           
                              

bool __file_peek_char(file *f, char *ch) {
/* Internal-File-Peek-Char
   Peeks at the next character in the file. If needed, update the file
   buffers. This function does not advance the file pointer by one
   character after the character is read. */

   sizea size;

   if(__file_is_readable(f)) {

      /* Read next character. Read another buffer from file if needed */
      if(__file_is_null(f)) {
         *ch = 0;
         return(true);
      } 
      if(__file_is_memory(f)) {
         if(f->bufptr < f->buf + f->bufsize) {
            *ch = *f->bufptr;
            return(true);
         }
         return(false);
      }

      /* Reading from a file */
      if(f->bufptr < f->buf + f->bufsize) {
         *ch = *f->bufptr;
         return(true);
      }

      /* Have exceeded file IO buffer */
      size = __file_read_buffer(f);
      if(size > 0) {
         f->bufptr = f->buf;
         f->bufsize = size;
         *ch = *f->bufptr;
         return(true);
      }

   }
   return(false);

}



bool file_peek_char(file *f, char *ch) {
/* File-Peek-Char */

   if(f != NULL && ch != NULL) return(__file_peek_char(f, ch));
   return(false);

}



bool __file_read_char(file *f, char *ch) {
/* Internal-File-Read-Char
   Reads a character from the file. The result is stored in <ch>. 
   Returns false on end of file. This function will also advance
   the file pointer by one character. */

   if(__file_peek_char(f, ch)) {
      __file_advance_line_column(f, *ch);
      f->bufptr++;
      return(true);
   }
   return(false);

}



bool file_read_char(file *f, char *ch) {
/* File-Read-Char */

   if(f != NULL && ch != NULL) return(__file_read_char(f, ch));
   return(false);

}



sizea file_read_line(file *f, char *rbuf, sizea size) {
/* File-Read-Line
   return buffer should be at least <size> bytes long. This functions
   advances the file pointer in <f>. Well, duh. The "line" is defined
   as a string of characters up until a newline or a NULL character.
   If a newline is reached, it is NOT returned. */

   sizea siz;
   
   if(f != NULL && rbuf != NULL && size > 0) {
      if(__file_peek_char(f, rbuf)) {
         siz = size;
         while(size > 1 && __file_read_char(f, rbuf) && (*rbuf != '\0' && *rbuf != '\n')) {
            rbuf++;
            size--;
         }
         *rbuf = 0;
         return(siz - size);
      }
   }
   return(-1);

}


      
sizea file_read_string(file *f, char *rbuf, sizea size) {
/* File-Read-String
   return buffer should be at least <size> bytes long. This functions
   advances the file pointer in <f>. Well, duh. The "string" is defined
   as a string of characters up until a NULL character. If a newline is
   reached, it is returned. */

   sizea siz;
   
   if(f != NULL && rbuf != NULL && size > 0) {
      if(__file_peek_char(f, rbuf)) {
         siz = size;
         while(size > 1 && __file_read_char(f, rbuf) && *rbuf != '\0') {
            rbuf++;
            size--;
         }
         *rbuf = 0;
         return(siz - size);
      }
   }
   return(-1);

}


      
sizea file_read_buffer(file *f, char *rbuf, sizea size) {
/* File-Read-Buffer
   return buffer should be at least <size> bytes long. This functions
   advances the file pointer in <f>. Well, duh. */

   sizea siz;

   if(f != NULL && rbuf != NULL && size > 0) {
      if(__file_peek_char(f, rbuf)) {
         siz = size;
         while(size > 0 && __file_read_char(f, rbuf)) {
            rbuf++;
            size--;
         }
         return(siz - size);
      }
   }
   return(-1);

}


      
bool file_read_word(file *f, word *ch) {
/* File-Read-Word */

   union _wordbytes {
      word value;
      char bytes[sizeof(word)];
   } data;
   
   if(f != NULL && ch != NULL) {
      if(__file_read_char(f, data.bytes) && __file_read_char(f, data.bytes + 1)) {
         *ch = data.value;
         return(true);
      }
   }
   return(false);

}


      
bool file_read_dword(file *f, dword *ch) {
/* File-Read-DWord */

   union _dwordbytes {
      dword value;
      char bytes[sizeof(dword)];
   } data;
   
   if(f != NULL && ch != NULL) {
      if(__file_read_char(f, data.bytes) && __file_read_char(f, data.bytes + 1) 
        && __file_read_char(f, data.bytes + 2) && __file_read_char(f, data.bytes + 3)) {
         *ch = data.value;
         return(true);
      }
   }
   return(false);

}


      
bool file_read_sword(file *f, sword *ch) {
/* File-Read-Signed-Word */

   return(file_read_word(f, (word *)ch));
   
}



bool file_read_sdword(file *f, sdword *ch) {
/* File-Read-Signed-DWord */

   return(file_read_dword(f, (dword *)ch));
   
}



