/* 

   libj - file_write.c           Copyright (C) 2001-1998 Justin David Smith    
   justins@chaos2.org            http://chaos2.org/
    
   File output


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#include <libjfile.h>
#include <_file_macro.h>



bool __file_write_buffer(file *f) {
/* Internal-File-Write-Buffer */

   sizea size;
   sizea num;

   if(__file_is_file(f)) {
      if(__file_is_writable(f) && !__file_buf_at_begin(f)) {
         size = f->bufptr - f->buf;
         if(f->func) {
            if(f->hooks.fwrite != NULL) num = f->hooks.fwrite(f, f->f, f->buf, size);
            else num = -1;
         } else num = write(f->f, f->buf, size);
         if(num == size) {
            __file_buf_set_begin(f);
            return(true);
         }
      }
   }
   return(false);

}



bool __file_write_char(file *f, char ch) { 
/* Internal-File-Write-Char */

   if(__file_is_writable(f)) {

      /* Write character to a memory buffer? */
      if(__file_is_null(f)) return(true);
      if(__file_is_memory(f)) {
         if(f->bufptr < f->buf + f->bufsize) {
            *f->bufptr = ch;
            f->bufptr++;
            __file_advance_line_column(f, ch);
            return(true);
         }
         return(false);
      }

      /* Writing to a file IO buffer */
      if(f->bufptr < f->buf + f->bufsize || __file_write_buffer(f)) {
         *f->bufptr = ch;
         f->bufptr++;
         __file_advance_line_column(f, ch);
         return(true);
      }
   
   }
   return(false);

}



bool file_write_char(file *f, char ch) { 
/* File-Write-Char */

   if(f != NULL) return(__file_write_char(f, ch));
   return(false);

}



bool file_write_line(file *f, const char *wbuf) { 
/* File-Write-Line */

   if(f != NULL) {
      while(*wbuf != '\0') {
         if(!__file_write_char(f, *wbuf)) return(false);
         wbuf++;
      }
      if(!__file_write_char(f, '\n')) return(false);
      return(true);
   }
   return(false);

};



bool file_write_string(file *f, const char *wbuf) { 
/* File-Write-String */

   if(f != NULL) {
      while(*wbuf != '\0') {
         if(!__file_write_char(f, *wbuf)) return(false);
         wbuf++;
      }
      if(!__file_write_char(f, '\0')) return(false);
      return(true);
   }
   return(false);

};



bool file_write_buffer(file *f, const char *wbuf, sizea size) { 
/* File-Write-Buffer */

   if(f != NULL) {
      while(size > 0 && __file_write_char(f, *wbuf)) {
         wbuf++;
         size--;
      }
      return(size == 0);
   }
   return(false);

};



bool file_write_word(file *f, word ch) {

   return(file_write_buffer(f, (const char *)&ch, sizeof(word)));

}



bool file_write_dword(file *f, dword ch) {

   return(file_write_buffer(f, (const char *)&ch, sizeof(dword)));

}



bool file_write_sword(file *f, sword ch) {

   return(file_write_buffer(f, (const char *)&ch, sizeof(word)));
   
}



bool file_write_sdword(file *f, sdword ch) {

   return(file_write_buffer(f, (const char *)&ch, sizeof(sdword)));

}
