/* $Header: /fridge/cvs/libj/jlink/_link.h,v 1.2 2001/12/10 19:43:59 justins Exp $ */                                                                                                    
/* 

   libj - _link.h                Copyright (C) 2001-1998 Justin David Smith    
   justins (a) chaos2.org        http://chaos2.org/
    
   Singly- and doubly-linked library class


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#ifndef __libj__link_included
#define __libj__link_included



static inline slinknode *__slink_alloc(void) {
/* slink_alloc */

   return((slinknode *)malloc(sizeof(slinknode)));
   
}



static inline slinknode *__dlink_alloc(void) {
/* slink_alloc */

   return((slinknode *)malloc(sizeof(dlinknode)));
   
}



static inline slinknode *__link_insert_after(slinknode *l, slinknode *newnode, void *d, destructorfn destr) {
/* link_insert_after

   Inserts data <d> after the current link node.  If the link node is NULL,
   then a new node will be created.  Otherwise, a node will be inserted
   immediately after <l>.  <destr> specifies the destructor for the data;
   it may be NULL (data cannot be destructed), or it may be a valid
   destructor for the data being added.  The new node is returned on
   success; otherwise, NULL is returned.  */
   
   /* Make sure newly allocated node isn't NULL */
   if(newnode != NULL) {
      newnode->data  = d;
      newnode->destr = destr;
      
      /* Attempt to install node after given node */
      if(l == NULL) {
         /* Creating a new node entirely */
         newnode->next = NULL;
      } else {
         /* Add this node after the current node */
         newnode->next = l->next;
         l->next = newnode;
      }
   }

   /* Return the new node */
   return(newnode);
      
}



static inline slinknode *__link_insert_before(slinknode *l, slinknode *newnode, void *d, destructorfn destr) {
/* link_insert_before

   Inserts data <d> before the current link node.  If the link node is NULL,
   then a new node will be created.  Otherwise, a node will be inserted
   immediately before <l>.  <destr> specifies the destructor for the data;
   it may be NULL (data cannot be destructed), or it may be a valid
   destructor for the data being added.  The new node is returned on
   success; otherwise, NULL is returned.  */
   
   /* Make sure newly allocated node isn't NULL */
   if(newnode != NULL) {
      newnode->data  = d;
      newnode->destr = destr;
      
      /* Setup new node to refer to the original node */
      newnode->next = l;
   }

   /* Return the new node */
   return(newnode);
      
}



static inline dlinknode *__link_head(dlinknode *l) {
/* __link_head

   Find the head (first element) of linked list <l>.  If the list is
   circular, then <l> is always considered the "head".  */
   
   dlinknode *cur = l;

   if(cur != NULL) {
      while(cur->prev != NULL && cur != l) {
         cur = cur->prev;
      }
   }
   return(cur);

}



static inline slinknode *__link_tail(slinknode *l) {
/* link_tail

   Find the tail (last element) of linked list <l>.  If the list is
   circular, then the element before <l> is returned.  */

   slinknode *cur = l;

   if(cur != NULL) {
      while(cur->next != NULL && cur->next != l) {
         cur = cur->next;
      }
   }
   return(cur);

}



static inline slinknode *__link_free(slinknode *l, const bool deletedata) {

   slinknode *next = l->next;
   if(deletedata && l->destr != NULL) l->destr(l->data);
   free(l);
   return(next);

}



#endif /* Header included? */
