/* $Header: /fridge/cvs/libj/jlink/libjlink.h,v 1.2 2001/12/10 19:43:59 justins Exp $ */                                                                                                    
/* 

   libj - jlink.h                Copyright (C) 2001-1998 Justin David Smith    
   justins (a) chaos2.org        http://chaos2.org/
    
   Header for linked lists


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#ifndef __libj_link_included
#define __libj_link_included



/* Required include files */
#include <libj.h>



/* Definition of a basic node */
typedef struct _slinknode {
   void *data;             /* Pointer to data */
   destructorfn destr;     /* Destructor function for data */
   struct _slinknode *next;/* Pointer to the next linked node */
} slinknode;



/* Definition of a basic doubly-linked node */
typedef struct _dlinknode {
   void *data;             /* Pointer to data */
   destructorfn destr;     /* Destructor function for data */
   struct _dlinknode *next;/* Pointer to the next linked node */
   struct _dlinknode *prev;/* Pointer to next dlinked node */
} dlinknode;



/* Function declarations */



#ifdef __cplusplus
extern "C" {
#endif /* C++? */


/* Functions for insertion/deletion */

slinknode *slink_insert_after(slinknode *l, void *d, destructorfn destr);
slinknode *slink_insert_before(slinknode *l, void *d, destructorfn destr);
slinknode *slink_append_list(slinknode *l, void *d, destructorfn destr);
slinknode *slink_prepend_list(slinknode *l, void *d, destructorfn destr);
slinknode *slink_append_head(slinknode **head, void *d, destructorfn destr);
slinknode *slink_prepend_head(slinknode **head, void *d, destructorfn destr);
#define slink_append(h, d, x)    slink_append_head((h), (d), (x))
#define slink_prepend(h, d, x)   slink_prepend_head((h), (d), (x))

dlinknode *dlink_insert_after(dlinknode *l, void *d, destructorfn destr);
dlinknode *dlink_insert_before(dlinknode *l, void *d, destructorfn destr);
dlinknode *dlink_append_list(dlinknode *l, void *d, destructorfn destr);
dlinknode *dlink_prepend_list(dlinknode *l, void *d, destructorfn destr);
dlinknode *dlink_append_head(dlinknode **head, void *d, destructorfn destr);
dlinknode *dlink_append_tail(dlinknode **tail, void *d, destructorfn destr);
dlinknode *dlink_prepend_head(dlinknode **head, void *d, destructorfn destr);
dlinknode *dlink_prepend_tail(dlinknode **tail, void *d, destructorfn destr);
#define dlink_append(h, d, x)    dlink_append_head((h), (d), (x))
#define dlink_prepend(h, d, x)   dlink_prepend_head((h), (d), (x))

void slink_free(slinknode **l, bool deletedata);
void dlink_free(dlinknode *l, bool deletedata);


/* General information for the list */

slinknode *slink_tail(slinknode *l);
bool slink_is_circular(const slinknode *l);
sizea slink_count(const slinknode *l);

dlinknode *dlink_head(dlinknode *l);
dlinknode *dlink_tail(dlinknode *l);
bool dlink_is_circular(const dlinknode *l);
sizea dlink_count(const dlinknode *l);
sizea dlink_count_back(const dlinknode *l);
sizea dlink_count_forward(const dlinknode *l);

#define link_is_head(l)       (link_prev(l) == NULL)
#define link_is_tail(l)       (link_next(l) == NULL)
#define link_is_empty(l)      ((l) == NULL)


/* Accessors/mutators for a single node */

slinknode *slink_get(slinknode *l, sizea index);
dlinknode *dlink_get(dlinknode *l, indexa index);
#define link_data(l)          ((l)->data)
#define link_next(l)          ((l) == NULL ? NULL : (l)->next)
#define link_prev(l)          ((l) == NULL ? NULL : (l)->prev)
slinknode *slink_replace(slinknode *l, void *d, destructorfn destr, bool deleteold);
dlinknode *dlink_replace(dlinknode *l, void *d, destructorfn destr, bool deleteold);


/* Manipulations on sublists */

slinknode *slink_split_after(slinknode *l);
slinknode *slink_merge_after(slinknode *l, slinknode *m);

dlinknode *dlink_split_before(dlinknode *l);
dlinknode *dlink_split_after(dlinknode *l);
dlinknode *dlink_merge_before(dlinknode *l, dlinknode *m);
dlinknode *dlink_merge_after(dlinknode *l, dlinknode *m);


/* Element manipulation, deletion */
slinknode *slink_iterate(slinknode *l, iteratorfn iter, void *params);
dlinknode *dlink_iterate(dlinknode *l, iteratorfn iter, void *params);
dlinknode *dlink_iterate_reverse(dlinknode *l, iteratorfn iter, void *params);

bool slink_delete(slinknode **l, const void *data, destructorfn destr, comparefn equal, bool deletedata);
bool slink_delete_all(slinknode **l, const void *data, destructorfn destr, comparefn equal, bool deletedata);
bool dlink_delete(dlinknode **l, const void *data, destructorfn destr, comparefn equal, bool deletedata);
bool dlink_delete_all(dlinknode **l, const void *data, destructorfn destr, comparefn equal, bool deletedata);


/* List deletion */

void slink_release(slinknode **l);
void dlink_release(dlinknode **l);
void slink_collapse(slinknode **l);
void dlink_collapse(dlinknode **l);


#ifdef __cplusplus
};    /* Close the extern */
#endif /* C++? */



#endif /* Header file is included */

