/* $Header: /fridge/cvs/libj/jlink/libjtree.h,v 1.3 2001/12/10 19:43:59 justins Exp $ */                                                                                                    
/* 

   libjlink - libjtree.h          Copyright (C) 2001-1998 Justin David Smith
   justins (a) chaos2.org         http://chaos2.org/
    
   Binary tree header


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#ifndef __libj_tree_included
#define __libj_tree_included



/* Required include files */
#include <libj.h>



/* Definition of function handlers */
typedef bool (*bintree_walkf)(const void *data, void *params);    /* Return nonzero to continue walking */


/* Definition of node colors, and the tree nodes */
typedef enum _bintreecolor {
   RED   =  true,
   BLACK =  false
} bintreecolor;

typedef struct _bintreenode {
   struct _bintreenode *l;    /* Left subtree (< this node) */
   struct _bintreenode *r;    /* Right tree (> this node) */
   struct _bintreenode *p;    /* Parent tree */
   bintreecolor c;            /* Color of node */
   void *data;                /* Pointer to data */
} bintreenode;

/* Definition of a binary tree */
typedef struct _bintree {
   struct _bintreenode *root; /* Root of tree */
   struct _bintreenode *sent; /* Sentinel (leafs = NULL) */
   comparefn lessthan;        /* Comparator function */
   destructorfn destr;        /* Destructor for data */
} bintree;

/* Pairs store a (character) key and value */
typedef struct _bintreepair {
   const char *key;           /* This is what sorting is done on */
   const char *value;         /* Arbitrary data stored with the key */
} bintreepair;



/* Function declarations */

#ifdef __cplusplus
extern "C" {
#endif /* C++? */


/*** Creation/Destruction functions ***/

bintree *bintree_new(comparefn lessthan, destructorfn destr);
void bintree_release(bintree **t);
void bintree_collapse(bintree **t);


/*** Insertion and deletion ***/

bool bintree_insert(bintree *t, void *data);
bool bintree_delete(bintree *t, const void *data, bool deletedata);


/*** Accessor functions ***/

sizea bintree_walk(const bintree *t, bintree_walkf w, void *params);
sizea bintree_walk_rev(const bintree *t, bintree_walkf w, void *params);
bool bintree_search(const bintree *t, const void *d, bintree_walkf w, void *params);
bool bintree_search_rev(const bintree *t, const void *d, bintree_walkf w, void *params);


/*** Generic Use Functions ***/

bool bintree_int_lessthan(const void *a, const void *b);
bool bintree_long_lessthan(const void *a, const void *b);
bool bintree_double_lessthan(const void *a, const void *b);
bool bintree_string_lessthan(const void *a, const void *b);
bool bintree_walk_count(const void *data, void *params);


/*** Search and detailed walk functions ***/

const bintreenode *bintree_find_first(const bintree *t, const void *d);
const bintreenode *bintree_prev(const bintree *t, const bintreenode *node);
const bintreenode *bintree_next(const bintree *t, const bintreenode *node);


/*** Miscellaneous functions ***/

int bintree_count(const bintree *t);


/*** Pair functions ***/

bintreepair *pair_alloc(const char *key, const char *value);
void pair_update_value(bintreepair *pair, const char *value);
bool bintree_pair_lessthan(const void *a, const void *b);
const bintreenode *bintree_pair_find_first(const bintree *t, const char *key);
bool bintree_pair_delete(bintree *t, const char *key, bool deletedata);
void pair_free(bintreepair *pair);


#ifdef __cplusplus
};    /* Close the extern */
#endif /* C++? */



#endif /* Header file is included */
