/* $Header: /fridge/cvs/libj/jlink/link.c,v 1.3 2001/12/10 19:43:59 justins Exp $ */                                                                                                    
/* 

   libj - link.c                 Copyright (C) 2001-1998 Justin David Smith    
   justins (a) chaos2.org        http://chaos2.org/
    
   Singly- and doubly-linked library class


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#include <libjlink.h>
#include <_link.h>



slinknode *slink_insert_after(slinknode *l, void *d, destructorfn destr) {
/* slink_insert_after */

   return(__link_insert_after(l, __slink_alloc(), d, destr));

}



dlinknode *dlink_insert_after(dlinknode *l, void *d, destructorfn destr) {
/* dlink_insert_after */

   dlinknode *newnode;

   newnode = (dlinknode *)__link_insert_after((slinknode *)l, __dlink_alloc(), d, destr);
   if(newnode != NULL) {
      /* Fixup the back pointers */
      newnode->prev = l;
      if(newnode->next != NULL) {
         newnode->next->prev = newnode;
      }
   }
   return(newnode);
      
}



slinknode *slink_insert_before(slinknode *l, void *d, destructorfn destr) {
/* slink_insert_before */

   return(__link_insert_before(l, __slink_alloc(), d, destr));
      
}



dlinknode *dlink_insert_before(dlinknode *l, void *d, destructorfn destr) {
/* dlink_insert_before */

   dlinknode *newnode;   

   /* Allocate memory for a new node */
   newnode = (dlinknode *)__link_insert_before((slinknode *)l, __dlink_alloc(), d, destr);

   /* Make sure newly allocated node isn't NULL, then fixup pointers */
   if(newnode != NULL) {

      /* Fixup back pointers on newnode and l */
      if(l != NULL) {
         newnode->prev = l->prev;
         l->prev = newnode;
      } else {
         newnode->prev = NULL;
      }

      /* Fixup the forward pointer on the previous node */
      if(newnode->prev != NULL) {
         newnode->prev->next = newnode;
      }
      
   }
   
   /* Return the new node */
   return(newnode);
      
}



slinknode *slink_append_list(slinknode *l, void *d, destructorfn destr) {
/* slink_append_list

   Inserts data <d> at the end of the given list.  If the link node is NULL,
   then a new node will be created.  Otherwise, a node will be inserted at
   the end of <l>.  <destr> specifies the destructor for the data; it may
   be NULL (data cannot be destructed), or it may be a valid destructor for
   the data being added.  The new node is returned on success; otherwise,
   NULL is returned.  */

   /* Walk to end of list, then create a node */
   return(__link_insert_after(__link_tail(l), __slink_alloc(), d, destr));
      
}



dlinknode *dlink_append_list(dlinknode *l, void *d, destructorfn destr) {
/* dlink_append_list

   Inserts data <d> at the end of the given list.  If the link node is NULL,
   then a new node will be created.  Otherwise, a node will be inserted at
   the end of <l>.  <destr> specifies the destructor for the data; it may
   be NULL (data cannot be destructed), or it may be a valid destructor for
   the data being added.  The new node is returned on success; otherwise,
   NULL is returned.  */

   slinknode *tail;
   dlinknode *newnode;

   /* Find end of list, and create a new node there */
   tail = __link_tail((slinknode *)l);
   newnode = (dlinknode *)__link_insert_after(tail, __dlink_alloc(), d, destr);
   if(newnode != NULL) {
      /* Fixup the back pointers */
      newnode->prev = (dlinknode *)tail;
      if(newnode->next != NULL) {
         newnode->next->prev = newnode;
      }
   }
   return(newnode);
      
}



slinknode *slink_prepend_list(slinknode *l, void *d, destructorfn destr) {
/* slink_prepend_list

   This is functionally identical to slink_insert_before().  */

   return(__link_insert_before(l, __slink_alloc(), d, destr));

}



dlinknode *dlink_prepend_list(dlinknode *l, void *d, destructorfn destr) {
/* link_prepend_list

   Inserts data <d> at the beginning of the given list.  If the link node is
   NULL, then a new node will be created.  Otherwise, a node will be
   inserted at the beginning of <l>.  <destr> specifies the destructor for
   the data; it may be NULL (data cannot be destructed), or it may be a
   valid destructor for the data being added.  The new node is returned on
   success; otherwise, NULL is returned.  */

   dlinknode *head;
   dlinknode *newnode;
   
   /* Walk to beginning of list, then create a node */
   head = __link_head(l);
   newnode = (dlinknode *)__link_insert_before((slinknode *)head, __dlink_alloc(), d, destr);

   /* Make sure newly allocated node isn't NULL, then fixup pointers */
   if(newnode != NULL) {

      /* Fixup back pointers on newnode and l */
      if(head != NULL) {
         newnode->prev = head->prev;
         head->prev = newnode;
      } else {
         newnode->prev = NULL;
      }

      /* Fixup the forward pointer on the previous node */
      if(newnode->prev != NULL) {
         newnode->prev->next = newnode;
      }
      
   }
   
   /* Return the new node */
   return(newnode);
      
}



slinknode *slink_append_head(slinknode **head, void *d, destructorfn destr) {
/* slink_append_head */

   slinknode *new;
   
   if(head != NULL) {
      /* Create the new linknode */
      new = slink_append_list(*head, d, destr);
      
      /* Only replace the head if the list was empty */
      if(new != NULL && *head == NULL) {
         *head = new;
      }
      
      /* Return the newly created node */
      return(new);
   }
   return(NULL);

}



dlinknode *dlink_append_head(dlinknode **head, void *d, destructorfn destr) {
/* dlink_append_head */

   dlinknode *new;
   
   if(head != NULL) {
      /* Create the new linknode */
      new = dlink_append_list(*head, d, destr);
      
      /* Only replace the head if the list was empty */
      if(new != NULL && *head == NULL) {
         *head = new;
      }
      
      /* Return the newly created node */
      return(new);
   }
   return(NULL);

}



slinknode *slink_prepend_head(slinknode **head, void *d, destructorfn destr) {
/* slink_prepend_head */

   slinknode *new;
   
   if(head != NULL) {
      /* Create the new linknode */
      new = slink_insert_before(*head, d, destr);
      
      /* Always replace the head */
      if(new != NULL) *head = new;
      
      /* Return the newly created node */
      return(new);
   }
   return(NULL);

}



dlinknode *dlink_prepend_head(dlinknode **head, void *d, destructorfn destr) {
/* dlink_prepend_head */

   dlinknode *new;
   
   if(head != NULL) {
      /* Create the new linknode */
      new = dlink_insert_before(*head, d, destr);
      
      /* Always replace the head */
      if(new != NULL) *head = new;
      
      /* Return the newly created node */
      return(new);
   }
   return(NULL);

}



dlinknode *dlink_append_tail(dlinknode **tail, void *d, destructorfn destr) {
/* dlink_append_tail */

   dlinknode *new;
   
   if(tail != NULL) {
      new = dlink_insert_after(*tail, d, destr);
      if(new != NULL) *tail = new;
      return(new);
   }
   return(NULL);

}



dlinknode *dlink_prepend_tail(dlinknode **tail, void *d, destructorfn destr) {
/* dlink_prepend_tail */

   dlinknode *new;
   
   if(tail != NULL) {
      new = dlink_prepend_list(*tail, d, destr);
      if(new != NULL && *tail == NULL) {
         *tail = new;
      }
      return(new);
   } 
   return(NULL);

}



void slink_free(slinknode **l, bool deletedata) {
/* slink_free

   Removes references to this link from the previous and next nodes (if they
   exist), and then deletes this node.  If this node contains data, and a
   destructor was specified for the data, and the <deletedata> flag is true,
   then the data will be deleted as well.  */

   if(l != NULL && *l != NULL) *l = __link_free(*l, deletedata);

}



void dlink_free(dlinknode *l, bool deletedata) {
/* dlink_free

   Removes references to this link from the previous and next nodes (if they
   exist), and then deletes this node.  If this node contains data, and a
   destructor was specified for the data, and the <deletedata> flag is true,
   then the data will be deleted as well.  */
   
   /* Nothing to do if the node is already NULL */
   if(l != NULL) {
   
      /* Remove references to this node from the next and previous pointers */
      if(l->next != NULL) l->next->prev = l->prev;
      if(l->prev != NULL) l->prev->next = l->next;
   
      /* Delete this node */
      __link_free((slinknode *)l, deletedata);
   }

}



dlinknode *dlink_head(dlinknode *l) {
/* dlink_head 

   Find the head (first element) of linked list <l>.  If the list is
   circular, then <l> is always considered the "head".  */

   return(__link_head(l));

}



slinknode *slink_tail(slinknode *l) {
/* slink_tail

   Find the tail (last element) of linked list <l>.  If the list is
   circular, then the element before <l> is returned.  */
   
   return(__link_tail(l));

}



dlinknode *dlink_tail(dlinknode *l) {
/* dlink_tail

   Find the tail (last element) of linked list <l>.  If the list is
   circular, then the element before <l> is returned.  */

   return((dlinknode *)__link_tail((slinknode *)l));

}



bool slink_is_circular(const slinknode *l) {
/* slink_is_circular

   Returns true if this linked list is circular.  A NULL list is not
   considered circular.  */
   
   const slinknode *cur;
   
   /* If list is null, always return false */
   if(l == NULL) return(false);
   
   /* Iterate until we reach self again, or reach end */
   cur = l->next;
   while(cur != NULL && cur != l) cur = cur->next;
   return(cur == l);
   
}

   
   
bool dlink_is_circular(const dlinknode *l) {
/* dlink_is_circular

   Returns true if this linked list is circular.  A NULL list is not
   considered circular.  */

   return(slink_is_circular((slinknode *)l));
   
}

   
   
sizea slink_count(const slinknode *l) {
/* slink_count

   Counts the number of entries in the linked list, forward of the current
   node.  Useful but try to cache this value as with large lists it can take
   some time to calculate this value.  If this list is circular, this will
   return the total number of elements in the list.  */

   const slinknode *cur;
   sizea count = 1;

   /* Count the entries! */
   if(l == NULL) return(0);
   cur = l->next;
   while(cur != NULL && cur != l) {
      count++;
      cur = cur->next;
   }
   return(count);

}



sizea dlink_count_forward(const dlinknode *l) {
/* dlink_count_forward

   Counts the number of entries in the linked list, forward of the current
   node.  Useful but try to cache this value as with large lists it can take
   some time to calculate this value.  If this list is circular, this will
   return the total number of elements in the list.  */

   const dlinknode *cur;
   sizea count = 1;

   /* Count the entries! */
   if(l == NULL) return(0);
   cur = l->next;
   while(cur != NULL && cur != l) {
      count++;
      cur = cur->next;
   }
   return(count);

}



sizea dlink_count_back(const dlinknode *l) {
/* dlink_count_back

   Counts the number of entries in the linked list, back from the current
   node.  Useful but try to cache this value as with large lists it can take
   some time to calculate this value.  If this list is circular, this will
   return the total number of elements in the list.  */

   const dlinknode *cur;
   sizea count = 1;

   /* Count the entries! */
   if(l == NULL) return(0);
   cur = l->prev;
   while(cur != NULL && cur != l) {
      count++;
      cur = cur->prev;
   }
   return(count);

}



sizea dlink_count(const dlinknode *l) {
/* dlink_count

   Counts the number of entries in the linked list.  Try to cache this value
   as with large lists it can take some time to calculate this value.  */

   const dlinknode *cur;
   sizea count = 1;

   /* Count the entries forward of this point */
   if(l == NULL) return(0);
   cur = l->next;
   while(cur != NULL && cur != l) {
      count++;
      cur = cur->next;
   }

   /* If list was circular, stop here */   
   if(cur == l) return(count);
   
   /* Start counting backward */
   cur = l->prev;
   while(cur != NULL) {
      count++;
      cur = cur->prev;
   }
   return(count);

}



slinknode *slink_get(slinknode *l, sizea index) {
/* slink_get

   Gets the <index>'th node in the linked list, where <l> is the zeroth
   entry.  Returns NULL if <index> is out of range.  You might find that for
   iterative methods, keeping a "current" pointer is far more efficient than
   repeated calls to this method.  */
   
   while(l != NULL && index > 0) {
      l = l->next;
      index--;
   }
   return(l);
   
}



dlinknode *dlink_get(dlinknode *l, indexa index) {
/* dlink_get

   Gets the <index>'th node in the linked list, where <l> is the zeroth
   entry (previous entries are negative).  Returns NULL if <index> is out of
   range.  You might find that for iterative methods, keeping a "current"
   pointer is far more efficient than repeated calls to this method.  */
   
   if(index >= 0) {
      while(l != NULL && index > 0) {
         l = l->next;
         index--;
      }
   } else {
      while(l != NULL && index < 0) {
         l = l->prev;
         index++;
      }
   }
   return(l);
   
}



slinknode *slink_replace(slinknode *l, void *d, destructorfn destr, bool deleteold) { 
/* slink_replace

   Replaces data in this node with the data specified, possibly destroying
   the old data (which is why this function may be more convenient than
   manipulating the data pointer directly).  If <l> is null, this is
   equivalent to an append operation.  */

   if(l != NULL) {
      /* Delete old data */
      if(deleteold && l->destr != NULL) l->destr(l->data);
   }
   
   /* Copy new data */
   l->data  = d;
   l->destr = destr;
   
   /* Return this node */
   return(l);
   
}



dlinknode *dlink_replace(dlinknode *l, void *d, destructorfn destr, bool deleteold) { 
/* dlink_replace */

   return((dlinknode *)slink_replace((slinknode *)l, d, destr, deleteold));
   
}



slinknode *slink_split_after(slinknode *l) {
/* slink_split_after

   Splits the list <l> after the current element.  <l> becomes the new tail
   of one list, and the pointer returned is the head of the other list. NULL
   is returned if <l> is NULL, or <l> is already the tail of a list.  */

   slinknode *newlist;

   /* Ignore if we are the tail of a list already */
   if(l != NULL) {
      newlist = l->next;
      l->next = NULL;
      return(newlist);
   }
   return(NULL);
   
}



dlinknode *dlink_split_before(dlinknode *l) {
/* dlink_split_before 

   Splits the list <l> before the current element.  <l> becomes the new head
   of one list, and the pointer returned is the TAIL of the other list. NULL
   is returned if <l> is NULL, or <l> is already the head of a list.  */

   dlinknode *newlist;

   /* Ignore if we are the head of a list already */
   if(l != NULL) {
      newlist = l->prev;
      l->prev = NULL;
      if(newlist != NULL) newlist->next = NULL;
      return(newlist);
   }
   return(NULL);
   
}



dlinknode *dlink_split_after(dlinknode *l) {
/* dlink_split_after

   Splits the list <l> after the current element.  <l> becomes the new tail
   of one list, and the pointer returned is the head of the other list. NULL
   is returned if <l> is NULL, or <l> is already the tail of a list.  */

   dlinknode *newlist;

   /* Ignore if we are the tail of a list already */
   if(l != NULL) {
      newlist = l->next;
      l->next = NULL;
      if(newlist != NULL) newlist->prev = NULL;
      return(newlist);
   }
   return(NULL);
   
}



slinknode *slink_merge_after(slinknode *l, slinknode *m) {
/* slink_merge_after

   Merges the entire list <m> after node <l>.  If <l> is NULL, then
   <m> is returned as-is.  If <m> is NULL, no action is taken.  <m>
   is returned in any case.  */

   slinknode *temp;
   
   /* If <l> or <m> NULL, then nothing to be done */
   if(l != NULL && m != NULL) {
   
      /* Now for the interesting case ... */
      temp = __link_tail(m);
      temp->next = l->next;
      l->next = m;

   }
   
   /* Return m */
   return(m);

}
   


dlinknode *dlink_merge_before(dlinknode *l, dlinknode *m) {
/* dlink_merge_before

   Merges the entire list <m> before node <l>.  If <l> is NULL, then
   <m> is returned as-is.  If <m> is NULL, no action is taken.  <m>
   is returned in any case.  */

   dlinknode *temp;
   
   /* If <l> or <m> NULL, then nothing to be done */
   if(l != NULL && m != NULL) {
   
      /* Now for the interesting case ... */
      temp = __link_head(m);
      temp->prev = l->prev;
      if(l->prev != NULL) l->prev->next = temp;
      temp = (dlinknode *)__link_tail((slinknode *)m);
      temp->next = l;
      l->prev = temp;
      
   }
   
   /* Return m */
   return(m);

}
   


dlinknode *dlink_merge_after(dlinknode *l, dlinknode *m) {
/* dlink_merge_after

   Merges the entire list <m> after node <l>.  If <l> is NULL, then
   <m> is returned as-is.  If <m> is NULL, no action is taken.  <m>
   is returned in any case.  */

   dlinknode *temp;
   
   /* If <l> or <m> NULL, then nothing to be done */
   if(l != NULL && m != NULL) {
   
      /* Now for the interesting case ... */
      temp = (dlinknode *)__link_tail((slinknode *)m);
      temp->next = l->next;
      if(l->next != NULL) l->next->prev = temp;
      temp = __link_head(m);
      temp->prev = l;
      l->next = temp;

   }
   
   /* Return m */
   return(m);

}
   


void slink_release(slinknode **l) {
/* slink-release

   This will go through and destroy, demolish, obliterate and annihilate an
   ENTIRE linked list from the specified node UP and DOWN.  This assumes the
   most generic type of linked list and will read the typeflag field to
   determine how to delete individual nodes. This function will also delete
   any data associated with each node if the LINKDELETABLE property is set
   in each node (in the typeflag field). If this property is not set or if
   dfunc is NULL, then the data associated with the node will not be
   deleted. This function returns true on success; should this function ever
   fail, you are seriously f*cked. Potential causes: linknode mismatch,
   serious errors while calling free(), or the custom dfunc() failed for
   some node. */

   slinknode *n;
   slinknode *del;

   /* Nothing to do if l == NULL */
   if(l != NULL) {

      /* Delete forward */
      n = *l;
      while(n != NULL) {
         del = n;
         n = n->next;
         __link_free(del, true);
      }
      
      *l = NULL;

   }

}



void slink_collapse(slinknode **l) {
/* slink_collapse

   Similar to linkRelease, except this function will not delete the data
   associated with each node (regardless of the LINKDELETABLE of <dfunc>
   flags in the node) */

   slinknode *n;
   slinknode *del;

   /* Nothing to do if l == NULL */
   if(l != NULL) {

      /* Delete forward */
      n = *l;
      while(n != NULL) {
         del = n;
         n = n->next;
         __link_free(del, false);
      }
      
      *l = NULL;

   }

}



void dlink_release(dlinknode **l) {
/* dlink-release

   This will go through and destroy, demolish, obliterate and annihilate an
   ENTIRE linked list from the specified node UP and DOWN.  This assumes the
   most generic type of linked list and will read the typeflag field to
   determine how to delete individual nodes. This function will also delete
   any data associated with each node if the LINKDELETABLE property is set
   in each node (in the typeflag field). If this property is not set or if
   dfunc is NULL, then the data associated with the node will not be
   deleted. This function returns true on success; should this function ever
   fail, you are seriously f*cked. Potential causes: linknode mismatch,
   serious errors while calling free(), or the custom dfunc() failed for
   some node. */

   dlinknode *n;
   dlinknode *del;

   /* Nothing to do if l == NULL */
   if(l != NULL && *l != NULL) {

      /* Delete in reverse (include current node) */
      n = (*l)->prev;
      while(n != NULL) {
         del = n;
         n = n->prev;
         __link_free((slinknode *)del, true);
      }
      
      /* Delete forward to end of list */
      slink_release((slinknode **)l);

   }

}



void dlink_collapse(dlinknode **l) {
/* dlink_collapse

   Similar to linkRelease, except this function will not delete the data
   associated with each node (regardless of the LINKDELETABLE of <dfunc>
   flags in the node) */

   dlinknode *n;
   dlinknode *del;

   /* Nothing to do if l == NULL */
   if(l != NULL && *l != NULL) {

      /* Delete in reverse (include current node) */
      n = (*l)->prev;
      while(n != NULL) {
         del = n;
         n = n->prev;
         __link_free((slinknode *)del, false);
      }
      
      /* Delete forward to end of list */
      slink_release((slinknode **)l);

   }

}
