/* $Header: /fridge/cvs/libj/jmem/libjmem.h,v 1.4 2001/12/11 05:04:20 justins Exp $ */                                                                                                    
/* 

   libj - jmem.h                 Copyright (C) 2001-1998 Justin David Smith    
   justins (a) chaos2.org        http://chaos2.org/
    
   General header file for memory library


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#ifndef __libj_mem_included
#define __libj_mem_included


/* Include system types */
#include <libj.h>
#include <string.h>


/* Definitions for memory return results */
#define  MEM_NO_MATCH            0xffffffff
#define  MEM_ARE_EQUAL(x)        !(x ^ MEM_NO_MATCH)
#define  MEM_ARE_UNEQUAL(x)      (x ^ MEM_NO_MATCH)


/* Disable libc functions unless requested */
#if (!ALLOW_LIBC_MEMORY)
   #undef      memset
   #define     memset         __dont_use_memset
   #undef      memmove
   #define     memmove        __dont_use_memmove
   #undef      memcmp
   #define     memcmp         __dont_use_memcmp
   #undef      memcpy
   #define     memcpy         __dont_use_memcpy
   #undef      bcmp
   #define     bcmp           __dont_use_bcmp
   #undef      bzero
   #define     bzero          __dont_use_bzero
   #undef      bcopy
   #define     bcopy          __dont_use_bcopy
#endif /* ALLOW_LIBC_MEMORY */


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*** Block memory relocation ***/
void *mmove(void *d, const void *s, sizea n);
void *mcopy(void *d, const void *s, sizea n);
void *mcopyb(void *d, const void *s, sizea n);
void *scopy(void *d, const void *s);
void *scopy_size(void *d, const void *s, sizea n);


/*** Scanning functions ***/
sizea mscan(const void *d, byte c, sizea limit);
sizea mscan_word(const void *d, word c, sizea limit);
sizea mscan_dword(const void *d, dword c, sizea limit);
sizea mscan_null(const void *d, sizea limit);
sizea sscan(const void *d, byte c);
sizea sscanr(const void *d, byte c);
sizea sscan_null(const void *d);


/*** Set functions ***/
void *mset(void *d, byte c, sizea n);
void *mset_word(void *d, word c, sizea n);
void *mset_dword(void *d, dword c, sizea n);
void *mset_null(void *d, sizea n);


/*** Block comparision ***/
sizea mcmp(const void *s, const void *d, sizea n);
sizea scmp(const void *s, const void *d);
sizea scmp_size(const void *s, const void *d, sizea n);


/*** Pattern searching ***/
void *rkmempat(void *s, const void *pat, sizea slen, sizea patlen);


/*** LIBC-style interface ***/
#define  jmemset(d, c, n)     mset((d), (c), (n))
#define  jmemcpy(d, s, n)     mcopy((d), (s), (n))
#define  jmemmove(d, s, n)    mmove((d), (s), (n))

static inline int __memcomp(const char *s, const char *d, sizea r) {
   if(MEM_ARE_EQUAL(r)) return(0);
   return(s[r] < d[r] ? -1 : 1);  
}
#define  jmemcmp(s, d, n)     __memcomp((s), (d), mcmp((s), (d), (n)))
#define  jmemeql(s, d, n)     (MEM_ARE_EQUAL(mcmp((s), (d), (n))))


#ifdef __cplusplus /* Close the extern declaration */
};
#endif /* __cplusplus */


#endif /* header included? */
