/* $Header: /fridge/cvs/libj/jmem/mem.c,v 1.2 2001/12/10 19:45:13 justins Exp $ */                                                                                                    
/* 

   libj - mem.c                  Copyright (C) 2001-1998 Justin David Smith
   justins (a) chaos2.org        http://chaos2.org/
    
   Memory manipulation functions


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/


/* Include files */
#ifdef LIBMEM_DEBUG
#include <stdlib.h>
#include <stdio.h>
#endif
#include <libjmem.h>


/*** Support functions ***/


#if LVALUE_STRICT
static inline const void *inc_ptr(const void **p) {
   byte *x = (byte *)*p;
   return((*p = (const void *)(++x)));
}


static inline const void *inc_ptr_word(const void **p) {
   word *x = (word *)*p;
   return((*p = (const void *)(++x)));
}


static inline const void *inc_ptr_dword(const void **p) {
   dword *x = (dword *)*p;
   return((*p = (const void *)(++x)));
}


static inline const void *add_to_ptr(const void **p, int n) {
   byte *x = (byte *)*p;
   return((*p = (const void *)(x + n)));
}


static inline const void *dec_ptr(const void **p) {
   byte *x = (byte *)*p;
   return((*p = (const void *)(--x)));
}


static inline const void *dec_ptr_word(const void **p) {
   word *x = (word *)*p;
   return((*p = (const void *)(--x)));
}


static inline const void *dec_ptr_dword(const void **p) {
   dword *x = (dword *)*p;
   return((*p = (const void *)(--x)));
}


static inline void *inc_ptr(void **p) {
   byte *x = (byte *)*p;
   return((*p = (void *)(++x)));
}


static inline void *inc_ptr_word(void **p) {
   word *x = (word *)*p;
   return((*p = (void *)(++x)));
}


static inline void *inc_ptr_dword(void **p) {
   dword *x = (dword *)*p;
   return((*p = (void *)(++x)));
}


static inline void *add_to_ptr(void **p, int n) {
   byte *x = (byte *)*p;
   return((*p = (void *)(x + n)));
}


static inline void *dec_ptr(void **p) {
   byte *x = (byte *)*p;
   return((*p = (void *)(--x)));
}


static inline void *dec_ptr_word(void **p) {
   word *x = (word *)*p;
   return((*p = (void *)(--x)));
}


static inline void *dec_ptr_dword(void **p) {
   dword *x = (dword *)*p;
   return((*p = (void *)(--x)));
}
#endif


/*** Memory block copy functions ***/


void *mmove(void *d, const void *s, sizea n) {
/* Memory-Move
   Copies a block of memory from s to d, of size n bytes. Is null-safe. 
   Returns NULL if s or d are NULL pointers. The buffers may overlap. */
   
   if(!s || !d) return(NULL);

   #if X86_ASM
   
      asm volatile ("
      
         # Welcome to the nightmare
         pushf
         movl     %0, %%esi
         movl     %1, %%edi
         movl     %2, %%ecx

         # Check to see if buffers overlap
         cmpl     %%edi, %%esi
         jb       mmove_dest_after_source
         
         # Destination is before source
         # forward copy is safe
         movl     %%ecx, %%eax
         shrl     $2, %%ecx
         andl     $3, %%eax
         cld
         rep      
         movsl
         movl     %%eax, %%ecx
         rep      
         movsb
         jmp      mmove_end_of_function
         
      mmove_dest_after_source:
         # Check to see if we will have to
         # do a reverse copy, then.
         movl     %%esi, %%eax
         addl     %%ecx, %%eax
         cmpl     %%eax, %%edi
         jb       mmove_dest_inside_source
         
         # Destination is far beyond source
         # forward copy is safe
         movl     %%ecx, %%eax
         shrl     $2, %%ecx
         andl     $3, %%eax
         cld
         rep      
         movsl
         movl     %%eax, %%ecx
         rep      
         movsb
         jmp      mmove_end_of_function
         
      mmove_dest_inside_source:
         # Damn. Destination is in source. We
         # must perform a reverse copy. Sigh.
         addl     %%ecx, %%esi
         addl     %%ecx, %%edi
         decl     %%esi
         decl     %%edi
         movl     %%ecx, %%eax
         andl     $1, %%ecx
         std
         rep
         movsb    
         decl     %%esi
         decl     %%edi
         movl     %%eax, %%ecx
         shrl     $1, %%ecx
         andl     $1, %%ecx
         rep
         movsw
         decl     %%esi
         decl     %%edi
         decl     %%esi
         decl     %%edi
         mov      %%eax, %%ecx
         shrl     $2, %%ecx
         rep
         movsl

      mmove_end_of_function:
         popf
         
      "
      :
      : "g" (s), "g" (d), "g" (n)
      : "eax", "ecx", "esi", "edi"
      );
      
   #else
      if(d <= s) {
         // Destination is before source buffer
         while(n & 0xfffffffc) {
            *((dword *)d) = *((dword *)s);
            #if LVALUE_STRICT
               inc_ptr_dword(&d);
               inc_ptr_dword(&s);
            #else
               ((dword *)d)++;
               ((dword *)s)++;
            #endif
            n -= 4;
         }
         if(n & 2) {
            *((word *)d) = *((word *)s);
            #if LVALUE_STRICT
               inc_ptr_word(&d);
               inc_ptr_word(&s);
            #else
               ((word *)d)++;
               ((word *)s)++;
            #endif
         }
         if(n & 1) {
            *((byte *)d) = *((byte *)s);
         }
      } else {
         if((byte *)d >= ((byte *)s) + n) {
            // destination is beyond source buffer
            while(n & 0xfffffffc) {
               *((dword *)d) = *((dword *)s);
               #if LVALUE_STRICT
                  inc_ptr_dword(&d);
                  inc_ptr_dword(&s);
               #else
                  ((dword *)d)++;
                  ((dword *)s)++;
               #endif
               n -= 4;
            }
            if(n & 2) {
               *((word *)d) = *((word *)s);
               #if LVALUE_STRICT
                  inc_ptr_word(&d);
                  inc_ptr_word(&s);
               #else
                  ((word *)d)++;
                  ((word *)s)++;
               #endif
            }
            if(n & 1) {
               *((byte *)d) = *((byte *)s);
            }
         } else {
            // Special case.. must copy memory _in_reverse_
            #if LVALUE_STRICT
               add_to_ptr(&d, n);
               add_to_ptr(&s, n);
            #else 
               ((byte *)d) += n;
               ((byte *)s) += n;
            #endif
            while(n & 0xfffffffc) {
               #if LVALUE_STRICT
                  dec_ptr_dword(&d);
                  dec_ptr_dword(&s);
               #else
                  ((dword *)d)--;
                  ((dword *)s)--;
               #endif
               n -= 4;
               *((dword *)d) = *((dword *)s);
            }
            if(n & 2) {
               #if LVALUE_STRICT
                  dec_ptr_word(&d);
                  dec_ptr_word(&s);
               #else
                  ((word *)d)--;
                  ((word *)s)--;
               #endif
               *((word *)d) = *((word *)s);
            }
            if(n & 1) {
               #if LVALUE_STRICT
                  dec_ptr(&d);
                  dec_ptr(&s);
               #else
                  ((byte *)d)--;
                  ((byte *)s)--;
               #endif
               *((byte *)d) = *((byte *)s);
            }
         }
      }
   #endif
   return(d);
   
}


void *mcopy(void *d, const void *s, sizea n) {
/* Memory-Copy
   Copies a block of memory from s to d, of size n bytes. Is null-safe. 
   Returns NULL if s or d are NULL pointers. This function is safe to use 
   if the buffers don't overlap or if d < s. */

   void *dstart = d;
   if(!s || !d) return(NULL);

   #if X86_ASM

      asm volatile ("
      
         # Welcome to the nightmare
         pushf
         movl     %0, %%esi
         movl     %1, %%edi
         movl     %2, %%ecx
         
         # forward copy
         movl     %%ecx, %%eax
         shrl     $2, %%ecx
         andl     $3, %%eax
         cld
         rep      
         movsl
         movl     %%eax, %%ecx
         rep      
         movsb
         popf
         
      "
      :
      : "g" (s), "g" (d), "g" (n)
      : "eax", "ecx", "esi", "edi"
      );
      
   #else
      while(n & 0xfffffffc) {
         *((dword *)d) = *((dword *)s);
         #if LVALUE_STRICT
            inc_ptr_dword(&d);
            inc_ptr_dword(&s);
         #else
            ((dword *)d)++;
            ((dword *)s)++;
         #endif
         n -= 4;
      }
      if(n & 2) {
         *((word *)d) = *((word *)s);
         #if LVALUE_STRICT
            inc_ptr_word(&d);
            inc_ptr_word(&s);
         #else
            ((word *)d)++;
            ((word *)s)++;
         #endif
      }
      if(n & 1) {
         *((byte *)d) = *((byte *)s);
      }
   #endif   
   return(dstart);
   
}


void *mcopyb(void *d, const void *s, sizea n) {
/* Memory-Copy-Back
   Copies a block of memory from s to d, of size n bytes. Is null-safe. 
   Returns NULL if s or d are NULL pointers. This function is safe to use 
   if the buffers don't overlap or if s < d. */

   void *dstart = d;
   if(!s || !d) return(NULL);

   #if X86_ASM

      asm volatile ("
      
         # Welcome to the nightmare
         pushf
         movl     %0, %%esi
         movl     %1, %%edi
         movl     %2, %%ecx

         # perform a reverse copy.
         addl     %%ecx, %%esi
         addl     %%ecx, %%edi
         decl     %%esi
         decl     %%edi
         movl     %%ecx, %%eax
         andl     $1, %%ecx
         std
         rep
         movsb    
         decl     %%esi
         decl     %%edi
         movl     %%eax, %%ecx
         shrl     $1, %%ecx
         andl     $1, %%ecx
         rep
         movsw
         decl     %%esi
         decl     %%edi
         decl     %%esi
         decl     %%edi
         mov      %%eax, %%ecx
         shrl     $2, %%ecx
         rep
         movsl
         popf
         
      "
      :
      : "g" (s), "g" (d), "g" (n)
      : "eax", "ecx", "esi", "edi"
      );
      
   #else
      #if LVALUE_STRICT
         add_to_ptr(&d, n);
         add_to_ptr(&s, n);
      #else 
         ((byte *)d) += n;
         ((byte *)s) += n;
      #endif
      while(n & 0xfffffffc) {
         #if LVALUE_STRICT
            dec_ptr_dword(&d);
            dec_ptr_dword(&s);
         #else
            ((dword *)d)--;
            ((dword *)s)--;
         #endif
         n -= 4;
         *((dword *)d) = *((dword *)s);
      }
      if(n & 2) {
         #if LVALUE_STRICT
            dec_ptr_word(&d);
            dec_ptr_word(&s);
         #else
            ((word *)d)--;
            ((word *)s)--;
         #endif
         *((word *)d) = *((word *)s);
      }
      if(n & 1) {
         #if LVALUE_STRICT
            dec_ptr(&d);
            dec_ptr(&s);
         #else
            ((byte *)d)--;
            ((byte *)s)--;
         #endif
         *((byte *)d) = *((byte *)s);
      }
   #endif   
   return(dstart);
   
}


void *scopy(void *d, const void *s) {
/* Copies a string until we encounter a NULL character in source string.
   This function is null safe, but d must be large enough to support the
   buffer s we will be copying. This function will write a NULL value to
   the end of the destination buffer. */
   
   void *dstart = d;
   if(!s || !d) return(NULL);

   #if X86_ASM__
      
      asm volatile ("
      
         # Welcome to the nightmare
         pushf
         movl     %0, %%esi
         movl     %1, %%edi
         movb     [%%esi], %%al
         orb      %%al, %%al
         jz       no_copy
         
      copy_loop:
         # forward copy until we encounter a NULL
         movb     %%al, [%%edi]
         movb     [%%esi], %%al
         incl     %%esi
         incl     %%edi
         orb      %%al, %%al
         jnz      copy_loop
         
      no_copy:
         # write the final NULL to the destination
         movb     0, [%%edi]
         popf
         
      "
      :
      : "g" (s), "g" (d)
      : "eax", "ecx", "esi", "edi"
      );
      
   #else
      while(*((byte *)s)) {
         *((byte *)d) = *((byte *)s);
         #if LVALUE_STRICT
            inc_ptr(&d);
            inc_ptr(&s);
         #else
            ((byte *)d)++;
            ((byte *)s)++;
         #endif
      }
      *((byte *)d) = 0;
   #endif
   return(dstart); 
     
}


void *scopy_size(void *d, const void *s, sizea n) {
/* Similar to scopy, except this function will copy at most n-1 characters from
   s to d. Thus, the destination buffer must be at least n characters
   long. */
   
   void *dstart = d;
   if(!s || !d) return(NULL);
   
   #ifdef LIBMEM_DEBUG_SCOPY
   printf("init: %s<-%p on %d chars\n", (char *)s, s, n);
   #endif
   
   n--;
   
   #if X86_ASM__
   
      asm volatile ("
      
         # Welcome to the nightmare
         pushf
         movl     %0, %%esi
         movl     %1, %%edi
         movl     %2, %%ecx
         movb     [%%esi], %%al
         orb      %%al, %%al
         jz       no_copy
         orb      %%ecx, %%ecx
         jz       no_copy
         
      copy_loop:
         # forward copy until we encounter a NULL
         movb     %%al, [%%edi]
         movb     [%%esi], %%al
         incl     %%esi
         incl     %%edi
         orb      %%al, %%al
         loopnz   copy_loop
         
      no_copy:
         # write the final NULL to the destination
         movb     0, [%%edi]
         popf
         
      "
      :
      : "g" (s), "g" (d), "g" (n)
      : "eax", "ecx", "esi", "edi"
      );
      
   #else
      #ifdef LIBMEM_DEBUG_SCOPY
      printf("beginning run... ");
      #endif
      while(n && *((byte *)s)) {
         #ifdef LIBMEM_DEBUG_SCOPY
         printf("<%c(%c)<-%p>", *((byte *)s), (char *)s, s);
         #endif
         *((byte *)d) = *((byte *)s);
         #if LVALUE_STRICT
            inc_ptr(&d);
            inc_ptr(&s);
         #else
            ((byte *)d)++;
            ((byte *)s)++;
         #endif
         n--;
      }
      *((byte *)d) = 0;
   #endif

   #ifdef LIBMEM_DEBUG_SCOPY
   printf("\nfinal: %s on %d chars\n", (char *)dstart, n);
   #endif

   return(dstart);
   
}


/*** Scanning functions ***/


sizea mscan(const void *d, byte c, sizea limit) {
/* This function scans for a <c> character in the buffer pointed to by
   <d>. This function will return the offset from d at which a character
   is found, or 0 if the pointer is not valid. If limit > 0, then it will
   scan at most <limit> characters. Returns 0xffffffff if we exceed the buffer
   sizea of <limit>. */

   #if !X86_ASM
   const byte *dscan = (const byte *)d;
   #endif
   sizea count = 0;
   if(!d) return(0);
   
   #if X86_ASM
   
      asm volatile ("
      
         # Moocow32
         pushf
         movl     %1, %%edi
         movl     %%edi, %%esi
         movb     %2, %%al
         movl     %3, %%ecx
         orl      %%ecx, %%ecx
         jz       mscan_limitless_scan

         # Limited scan for the byte in question
         cld
         repnz
         scasb
         jnz      mscan_not_found
         
         # The character was found at offset (edi - esi - 1)
         subl     %%esi, %%edi
         decl     %%edi
         movl     %%edi, %0
         jmp      mscan_end_of_function

      mscan_limitless_scan:
         # This is similar to the above scan, but we reinit ecx to max value
         movl     0xffffffff, %%ecx
         cld
         repnz
         scasb
         jnz      mscan_not_found
         
         # The character was found at offset (edi - esi - 1)
         subl     %%esi, %%edi
         decl     %%edi
         movl     %%edi, %0
         jmp      mscan_end_of_function
         
      mscan_not_found:
         # The search character was not found (?!)
         movl     0xffffffff, %0
         
      mscan_end_of_function:
         popf
         
      "
      : "g=" (count)
      : "g" (d), "g" (c), "g" (limit)
      : "eax", "ecx", "esi", "edi" );
      
   #else
      if(limit) {
         while(limit && *dscan != c) dscan++;
         if(limit) count = (sizea)((byte *)dscan - (byte *)d);
         else count = 0xffffffff;
      } else {
         while(*dscan != c) dscan++;
         count = (sizea)((byte *)dscan - (byte *)d);
      }
   #endif
   return(count);

}


sizea mscan_word(const void *d, word c, sizea limit) {
/* This function scans for a <c> word in the buffer pointed to by
   <d>. This function will return the offset from d at which a word
   is found, or 0 if the pointer is not valid. If limit > 0, then it will
   scan at most <limit> words. Returns 0xffffffff if we exceed the buffer
   size of <limit>. */

   #if !X86_ASM
   const word *dscan = (const word *)d;
   #endif
   sizea count = 0;
   if(!d) return(0);
   
   #if X86_ASM
   
      asm volatile ("
      
         # Moocow32
         pushf
         movl     %1, %%edi
         movl     %%edi, %%esi
         movw     %2, %%ax
         movl     %3, %%ecx
         orl      %%ecx, %%ecx
         jz       mscan_word_limitless_scan

         # Limited scan for the byte in question
         cld
         repnz
         scasw
         jnz      mscan_word_not_found
         
         # The character was found at offset (edi - esi - 1)
         subl     %%esi, %%edi
         shrl     $1, %%edi
         decl     %%edi
         movl     %%edi, %0
         jmp      mscan_word_end_of_function

      mscan_word_limitless_scan:
         # This is similar to the above scan, but we reinit ecx to max value
         movl     0xffffffff, %%ecx
         cld
         repnz
         scasw
         jnz      mscan_word_not_found
         
         # The character was found at offset (edi - esi - 1)
         subl     %%esi, %%edi
         shrl     $1, %%edi
         decl     %%edi
         movl     %%edi, %0
         jmp      mscan_word_end_of_function
         
      mscan_word_not_found:
         # The search character was not found (?!)
         movl     0xffffffff, %0
         
      mscan_word_end_of_function:
         popf
         
      "
      : "g=" (count)
      : "g" (d), "g" (c), "g" (limit)
      : "eax", "ecx", "esi", "edi" );
      
   #else
      if(limit) {
         while(limit && *dscan != c) dscan++;
         if(limit) count = (sizea)((word *)dscan - (word *)d);
         else count = 0xffffffff;
      } else {
         while(*dscan != c) dscan++;
         count = (sizea)((word *)dscan - (word *)d);
      }
   #endif
   return(count);

}


sizea mscan_dword(const void *d, dword c, sizea limit) {
/* This function scans for a <c> dword in the buffer pointed to by
   <d>. This function will return the offset from d at which a dword
   is found, or 0 if the pointer is not valid. If limit > 0, then it will
   scan at most <limit> dwords. Returns 0xffffffff if we exceed the buffer
   size of <limit>. */

   #if !X86_ASM
   const dword *dscan = (const dword *)d;
   #endif
   sizea count = 0;
   if(!d) return(0);
   
   #if X86_ASM
   
      asm volatile ("
      
         # Moocow32
         pushf
         movl     %1, %%edi
         movl     %%edi, %%esi
         movl     %2, %%eax
         movl     %3, %%ecx
         orl      %%ecx, %%ecx
         jz       mscan_dword_limitless_scan

         # Limited scan for the byte in question
         cld
         repnz
         scasl
         jnz      mscan_dword_not_found
         
         # The character was found at offset (edi - esi - 1)
         subl     %%esi, %%edi
         shrl     $2, %%edi
         decl     %%edi
         movl     %%edi, %0
         jmp      mscan_dword_end_of_function

      mscan_dword_limitless_scan:
         # This is similar to the above scan, but we reinit ecx to max value
         movl     0xffffffff, %%ecx
         cld
         repnz
         scasl
         jnz      mscan_dword_not_found
         
         # The character was found at offset (edi - esi - 1)
         subl     %%esi, %%edi
         shrl     $2, %%edi
         decl     %%edi
         movl     %%edi, %0
         jmp      mscan_dword_end_of_function
         
      mscan_dword_not_found:
         # The search character was not found (?!)
         movl     0xffffffff, %0
         
      mscan_dword_end_of_function:
         popf
         
      "
      : "g=" (count)
      : "g" (d), "g" (c), "g" (limit)
      : "eax", "ecx", "esi", "edi" );
      
   #else
      if(limit) {
         while(limit && *dscan != c) dscan++;
         if(limit) count = (sizea)((dword *)dscan - (dword *)d);
         else count = 0xffffffff;
      } else {
         while(*dscan != c) dscan++;
         count = (sizea)((dword *)dscan - (dword *)d);
      }
   #endif
   return(count);

}


sizea mscan_null(const void *d, sizea limit) {
/* This function scans for a NULL character in the buffer pointed to by
   <d>. This function will return the offset from d at which a null character
   is found, or 0 if the pointer is not valid. If limit > 0, then it will
   scan at most <limit> characters. Returns 0xffffffff if we exceed the buffer
   size of <limit>. */

   #if !X86_ASM
   const byte *dscan = (const byte *)d;
   #endif
   sizea count = 0;
   if(!d) return(0);
   
   #if X86_ASM
   
      asm volatile ("
      
         # Moocow32
         pushf
         movl     %1, %%edi
         movl     %%edi, %%esi
         movl     %2, %%ecx
         xorl     %%eax, %%eax
         orl      %%ecx, %%ecx
         jz       mscan_null_limitless_scan

         # Limited scan for the byte in question
         cld
         repnz
         scasb
         jnz      mscan_null_not_found
         
         # The character was found at offset (edi - esi - 1)
         subl     %%esi, %%edi
         decl     %%edi
         movl     %%edi, %0
         jmp      mscan_null_end_of_function

      mscan_null_limitless_scan:
         # This is similar to the above scan, but we reinit ecx to max value
         movl     0xffffffff, %%ecx
         cld
         repnz
         scasb
         jnz      mscan_null_not_found
         
         # The character was found at offset (edi - esi - 1)
         subl     %%esi, %%edi
         decl     %%edi
         movl     %%edi, %0
         jmp      mscan_null_end_of_function
         
      mscan_null_not_found:
         # The search character was not found (?!)
         movl     0xffffffff, %0
         
      mscan_null_end_of_function:
         popf
         
      "
      : "g=" (count)
      : "g" (d), "g" (limit)
      : "eax", "ecx", "esi", "edi" );
      
   #else
      if(limit) {
         while(limit && *dscan) dscan++;
         if(limit) count = (sizea)((byte *)dscan - (byte *)d);
         else count = 0xffffffff;
      } else {
         while(*dscan) dscan++;
         count = (sizea)((byte *)dscan - (byte *)d);
      }
   #endif
   return(count);

}


sizea sscan(const void *d, byte c) {
/* This function scans for a <c> character in the buffer pointed to by
   <d>. This function will return the offset from d at which a character
   is found, or 0 if the pointer is not valid. Returns 0xffffffff if we
   encounter a NULL character before finding the character. */

   const byte *dscan = (const byte *)d;
   sizea count = 0;
   if(!d) return(0);
   
   #if X86_ASM____
   
      #error No code X86_ASM
      
   #else
      while(*dscan && *dscan != c) dscan++;
      if(*dscan) {
         count = (sizea)((byte *)dscan - (byte *)d);
      } else count = 0xffffffff;
   #endif
   return(count);

}


sizea sscanr(const void *s, byte c) {

   const void *t = s;
   sizea sc;
   
   while((sc = sscan(t, c)) != MEM_NO_MATCH) t = (byte *)t + (sc + 1);
   if(t == s) return(MEM_NO_MATCH);
   return((sizea)((byte *)t - (byte *)s));

}


sizea sscan_null(const void *d) {
/* This function scans for a NULL character in the buffer pointed to by
   <d>. This function will return the offset from d at which a null character
   is found, or 0 if the pointer is not valid. */

   const byte *dscan = (const byte *)d;
   sizea count = 0;
   if(!d) return(0);
   
   #if X86_ASM__
   
      asm volatile ("
      
         # Moocow32
         pushf
         movl     %1, %%edi
         movl     %%edi, %%esi
         xorl     %%eax, %%eax
         movl     0xffffffff, %%ecx
         cld
         repnz
         scasb
         subl     %%esi, %%edi
         decl     %%edi
         movl     %%edi, %0
         popf
         
      "
      : "g=" (count)
      : "g" (d)
      : "eax", "ecx", "esi", "edi" );
      
   #else
      while(*dscan) dscan++;
      count = (sizea)((byte *)dscan - (byte *)d);
   #endif
   return(count);

}


/*** Memory set functions ***/


void *mset_null(void *d, sizea n) {
/* Clears the memory buffer pointed to by <d> with <n> NULL characters. */

   void *dstart = d;
   if(!d) return(NULL);

   #if X86_ASM
   
      asm volatile ("

         # Welcome to the Real World(moocow64)
         pushf
         movl     %0, %%edi
         movl     %1, %%ecx
         xorl     %%eax, %%eax
         movl     %%ecx, %%edx
         shrl     $2, %%ecx
         andl     $3, %%edx
         cld
         rep      
         stosl
         mov      %%edx, %%ecx
         rep
         stosb
         popf
         
      "
      :
      : "g" (d), "g" (n)
      : "edi", "ecx", "eax", "edx"
      );
      
   #else
      while(n & 0xfffffffc) {
         *((dword *)d) = 0;
         #if LVALUE_STRICT
            inc_ptr_dword(&d);
         #else
            ((dword *)d)++;
         #endif
         n -= 4;
      }
      if(n & 2) {
         *((word *)d) = 0;
         #if LVALUE_STRICT
            inc_ptr_word(&d);
         #else
            ((word *)d)++;
         #endif
      }
      if(n & 1) {
         *((byte *)d) = 0;
      }
   #endif
   return(dstart);

}


void *mset(void *d, byte c, sizea n) {
/* Initialises the memory buffer pointed to by the <d> pointer with <n> 
   characters, using <c> for the fill. */

   void *dstart = d;
   if(!d) return(NULL);

   #if X86_ASM____
   
      #error No code for mset X86_ASM
      
   #else
      while(n) {
         *((byte *)d) = c;
         #if LVALUE_STRICT
            inc_ptr(&d);
         #else
            ((byte *)d)++;
         #endif
         n--;
      }
   #endif
   return(dstart);

}


void *mset_word(void *d, word c, sizea n) {
/* Initialises the memory buffer pointed to by the <d> pointer with <n> 
   words, using <c> for the fill. */

   void *dstart = d;
   if(!d) return(NULL);

   #if X86_ASM____
   
      #error No code for mset X86_ASM
      
   #else
      while(n) {
         *((word *)d) = c;
         #if LVALUE_STRICT
            inc_ptr_word(&d);
         #else
            ((word *)d)++;
         #endif
         n--;
      }
   #endif
   return(dstart);

}


void *mset_dword(void *d, dword c, sizea n) {
/* Initialises the memory buffer pointed to by the <d> pointer with <n> 
   double-words, using <c> for the fill. */

   void *dstart = d;
   if(!d) return(NULL);

   #if X86_ASM____
   
      #error No code for mset X86_ASM
      
   #else
      while(n) {
         *((dword *)d) = c;
         #if LVALUE_STRICT
            inc_ptr_dword(&d);
         #else 
            ((dword *)d)++;
         #endif
         n--;
      }
   #endif
   return(dstart);

}


/*** Memory compare functions ***/


sizea mcmp(const void *s, const void *d, sizea n) {
/* This function compares n bytes in the two buffers specified, searching
   for the first different character. If the buffers contain identical
   data, then 0xffffffff is returned, otherwise the offset into each buffer
   of the first different character is returned. */
   
   sizea count = 0xffffffff;
   const void *start = s;
   if(!s || !d) return(0);
   
   #if X86_ASM____
   
      #error No code
      
   #else
      while(n) {
         if(*((byte *)s) != *((byte *)d)) {
            // Characters do not match
            n = 0;
            count = (sizea)((byte *)s - (byte *)start);
         } else {
            n--;
            #if LVALUE_STRICT
               inc_ptr(&s);
               inc_ptr(&d);
            #else
               ((byte *)s)++;
               ((byte *)d)++;
            #endif
         }
      }
   #endif
   return(count);

}


sizea scmp(const void *s, const void *d) {
/* This function compares bytes in the two strings specified, searching
   for the first different character. If the buffers contain identical
   data, then 0xffffffff is returned, otherwise the offset into each buffer
   of the first different character is returned. This function assumes each
   string is terminated by a null byte. */
   
   sizea count = 0xffffffff;
   const void *start = s;
   if(!s || !d) return(0);
   
   #if X86_ASM____
   
      #error No code
      
   #else
      while(*((byte *)s) && *((byte *)d) && !(count ^ 0xffffffff)) {
         if(*((byte *)s) != *((byte *)d)) {
            // Characters do not match
            count = (sizea)((byte *)s - (byte *)start);
         } else {
            #if LVALUE_STRICT
               inc_ptr(&s);
               inc_ptr(&d);
            #else
               ((byte *)s)++;
               ((byte *)d)++;
            #endif
         }
      }
      if(!(count ^ 0xffffffff)) {
         // Make sure both strings have terminated
         if(*((byte *)s) || *((byte *)d)) count = (sizea)((byte *)s - (byte *)start);
      }
   #endif
   return(count);

}


sizea scmp_size(const void *s, const void *d, sizea n) {
/* This function compares n bytes in the two strings specified, searching
   for the first different character. If the buffers contain identical
   data, then 0xffffffff is returned, otherwise the offset into each buffer
   of the first different character is returned. This function assumes each
   string is terminated by a null byte. */
   
   sizea count = 0xffffffff;
   const void *start = s;
   if(!s || !d) return(0);
   
   #if X86_ASM____
   
      #error No code
      
   #else
      while(*((byte *)s) && *((byte *)d) && n) {
         if(*((byte *)s) != *((byte *)d)) {
            // Characters do not match
            n = 0;
            count = (sizea)((byte *)s - (byte *)start);
         } else {
            n--;
            #if LVALUE_STRICT
               inc_ptr(&s);
               inc_ptr(&d);
            #else
               ((byte *)s)++;
               ((byte *)d)++;
            #endif
         }
      }
      if(n) {
         // Make sure both strings have terminated
         if(*((byte *)s) || *((byte *)d)) count = (sizea)((byte *)s - (byte *)start);
      }
   #endif
   return(count);

}


/*** Pattern Searching ***/


void *rkmempat(void *s, const void *pat, sizea slen, sizea patlen) {
/* Rabin-Karp-Memory-Pattern
   Searches for memory data in <pat> of length <patlen> in the target
   buffer <s>, of maximum length <slen>. Function will give up if <slen>
   is zero. */

   unsigned int i = 0;
   int p = 0;
   int t = 0;
   int h = 1;
   byte *sp = (byte *)s;
   const byte *patp = (byte *)pat;
   if(!s || !pat || !slen) return(NULL);
   if(slen < patlen) return(NULL);

   i = patlen;
   while(i) {
      t = ((t << 8) + *sp);
      if(t >= RK_PRIME) t %= RK_PRIME;
      p = ((p << 8) + *patp);
      if(p >= RK_PRIME) p %= RK_PRIME;
      sp++;
      patp++;
      i--;
   }

   // Precalc the infamous "h" value.
   if(patlen) {
      i = patlen - 1;
      while(i) {
         h = (h << 8);
         if(h >= RK_PRIME) h %= RK_PRIME;
         i--;
      }
   }

   // Pattern matching loop
   i = (slen - patlen) + 1;
   sp = (byte *)s;
   while(i) {
      if(p == t) {
         if(!(mcmp(sp, pat, patlen) ^ 0xffffffff)) return((void *)sp);
      }
      i--;
      if(i) {
         t = (((t - (*sp * h)) << 8) + *(sp + patlen)) % RK_PRIME;
         if(t < 0) t += RK_PRIME;
      }
      sp++;
   }
   return(NULL);

}


