/* 

   libj - _mtx_int.h             Copyright (C) 2001-1998 Justin David Smith
   justins@chaos2.org            http://chaos2.org/
    
   Internal header for matrix library


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#ifndef __libj__mtx_int_included
#define __libj__mtx_int_included



/* Matrix looping */

#define  MTX_LOOP_SIZEP(m,size,p)((size) = (m)->size, (p) = (m)->data)
#define  MTX_LOOP_ASSIGN(size,p,value) \
                                 while(size) {           \
                                    *(p) = (value);      \
                                    (p)++;               \
                                    (size)--;            \
                                 }
#define  MTX_LOOP2_ASSIGN(sizeh,sizew,p,skip,colsvar,value) \
                                 while(sizeh) {          \
                                    (colsvar) = (sizew); \
                                    MTX_LOOP_ASSIGN(colsvar, p, value); \
                                    (p) += (skip);       \
                                    (sizeh)--;           \
                                 }



/* Numerical manipulation */

#define  MTX_SWAP(a,b)           ((a) = (a) + (b), \
                                  (b) = (a) - (b), \
                                  (a) = (a) - (b))
#define  MTX_MIN(a,b)            ((a) < (b) ? (a) : (b))
#define  MTX_MAX(a,b)            ((a) > (b) ? (a) : (b))



/* Matrix type manipulators */

#define  MTX_IS_LONG(m)          ((m)->type & MTX_LONG)
#define  MTX_IS_DOUBLE(m)        ((m)->type & MTX_DOUBLE)
#define  MTX_IS_SPARSE(m)        ((m)->type & MTX_SPARSE)
#define  MTX_IS_GRID(m)          ((m)->type & MTX_GRID)
#define  MTX_IS_GLONG(m)         (MTX_IS_LONG(m) && MTX_IS_GRID(m))
#define  MTX_IS_GDOUBLE(m)       (MTX_IS_LONG(m) && MTX_IS_GRID(m))
#define  MTX_IS_CUSTOM(m)        ((m)->type & MTX_CUSTOM)



/* XYXY and XYWH bounds */

#define  MTX_IN_BOUNDS_WH(x,y,w,h)           (x >= 0 && y >= 0 && x < w && y < h)
#define  MTX_XYXY_IN_BOUNDS(x1,y1,x2,y2,width,height) \
                                             (MTX_IN_BOUNDS_WH(x1, y1, width, height) && \
                                              x2 - x1 >= 0 && y2 - y1 >= 0)
#define  MTX_XYWH_IN_BOUNDS(x,y,w,h,width,height) \
                                             (MTX_IN_BOUNDS_WH(x, y, width, height) && \
                                              w > 0 && h > 0)

#define  MTX_LEFT_CLIP_XYXY(x1,x2,width)     if(x1 < 0) x1 = 0
#define  MTX_TOP_CLIP_XYXY(y1,y2,height)     if(y1 < 0) y1 = 0
#define  MTX_RIGHT_CLIP_XYXY(x1,x2,width)    if(x2 >= width) x2 = width - 1
#define  MTX_BOTTOM_CLIP_XYXY(y1,y2,height)  if(y2 >=height) y2 = height- 1

#define  MTX_LEFT_CLIP_XYWH(x,w,width)       if(x < 0) w += x, x = 0
#define  MTX_TOP_CLIP_XYWH(y,h,height)       if(y < 0) h += y, y = 0
#define  MTX_RIGHT_CLIP_XYWH(x,w,width)      if(x + w > width) w = width - x
#define  MTX_BOTTOM_CLIP_XYWH(y,h,height)    if(y + h >height) h = height- y

#define  MTX_LEFT_SHIFT_XYXY(x1,x2,width)    if(x1 < 0) x2 -= x1, x1 = 0
#define  MTX_TOP_SHIFT_XYXY(y1,y2,height)    if(y1 < 0) y2 -= y1, y1 = 0
#define  MTX_RIGHT_SHIFT_XYXY(x1,x2,width)   if(x2 >= width) x1 -= (x2 - width + 1), x2 = width - 1
#define  MTX_BOTTOM_SHIFT_XYXY(y1,y2,height) if(y2 >=height) y1 -= (y2 -height + 1), y2 = height- 1

#define  MTX_LEFT_SHIFT_XYWH(x,w,width)      if(x < 0) x = 0
#define  MTX_TOP_SHIFT_XYWH(y,h,height)      if(y < 0) y = 0
#define  MTX_RIGHT_SHIFT_XYWH(x,w,width)     if(x + w > width) x = width - w;
#define  MTX_BOTTOM_SHIFT_XYWH(y,h,height)   if(y + h >height) y = height- h;
                  


/* Matrix informational */

#define  MTX_IS_SIZE_0(m)        ((m)->width == 0 || (m)->height == 0)
#define  MTX_IS_SIZE_1x1(m)      ((m)->width == 1 && (m)->height == 1)
#define  MTX_IS_SIZE_1xn(m)      ((m)->height== 1)
#define  MTX_IS_SIZE_nx1(m)      ((m)->width == 1)
#define  MTX_IS_SQUARE(m)        (MTX_IS_SIZE_0(m) || (m)->width == (m)->height)
#define  MTX_IS_VECTOR(m)        (MTX_IS_SIZE_1xn(m) || MTX_IS_SIZE_nx1(m))
#define  MTX_IS_VECTOR_HORIZ(m)  (MTX_IS_SIZE_nx1(m))
#define  MTX_IS_VECTOR_VERT(m)   (MTX_IS_SIZE_1xn(m))
#define  MTX_ARE_COMPAT(a,b)     ((a)->type == (b)->type && (a)->dtsize == (b)->dtsize)
#define  MTX_SAME_SIZE(a, b)     ((a)->width == (b)->width && (a)->height == (b)->height)
#define  MTX_VECTOR_SIZE(v)      ((v)->width == 1 ? (v)->height : (v)->width)



/* File I/O Version codes and magic numbers */

#define  GMTX_MAGIC              0x58746d47
#define  GMTX_VERSION_MIN        1
#define  GMTX_VERSION_MAX        1
#define  GMTX_MRKR_INFO          0x6f666e69
#define  GMTX_MRKR_LINE          0x656e696c
#define  GMTX_MRKR_END           0x20646e65



#endif
