/* 

   libj - gmtx.c                 Copyright (C) 2001-1998 Justin David Smith
   justins@chaos2.org            http://chaos2.org/
    
   Matrix information and comparison


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#include <libjmtx.h>
#include <_mtx_int.h>
#include <libjmem.h>
#include <libjstr.h>



/*** Matrix information and Comparison ***/
/* Internal functions */



static inline bool _gmtx_is_null(const gmtx *m) {

   /* If width = 0 or height = 0, then matrix is NULL */
   return(MTX_IS_SIZE_0(m));

}
   


static inline bool _gmtx_is_vector(const gmtx *m) {

   /* If width = 1 or height = 1, then matrix is a valid vector */
   return(MTX_IS_VECTOR(m));
   
}



static inline bool _gmtx_are_equal(const gmtx *a, const gmtx *b) {

   /* Check for NULL matrices first */
   if(_gmtx_is_null(a)) return(_gmtx_is_null(b));
   if(_gmtx_is_null(b)) return(0);
   
   /* Make sure attributes compare */
   if(!MTX_ARE_COMPAT(a, b))  return(0);
   if(!MTX_SAME_SIZE(a, b)) return(0);
   
   /* Compare the raw memory */
   return(jmemeql(a->data, b->data, a->bufsize));

}



static inline bool _gmtx_is_square(const gmtx *m) {

   /* By convention, a NULL matrix is square */
   return(MTX_IS_SQUARE(m));
   
}



/*** Matrix information and Comparison ***/
/* Interface functions */



bool gmtxl_is_null(const gmtxl *m) { return(_gmtx_is_null(__gmtxp m)); }
bool gmtxd_is_null(const gmtxd *m) { return(_gmtx_is_null(__gmtxp m)); }
bool gmtx_is_null(const gmtx *m)   { return(_gmtx_is_null(m)); }

bool gmtxl_is_vector(const gmtxl *m) { return(_gmtx_is_vector(__gmtxp m)); }
bool gmtxd_is_vector(const gmtxd *m) { return(_gmtx_is_vector(__gmtxp m)); }
bool gmtx_is_vector(const gmtx *m)   { return(_gmtx_is_vector(m)); }

bool gmtxl_are_equal(const gmtxl *a, const gmtxl *b) { return(_gmtx_are_equal(__gmtxp a, __gmtxp b)); }
bool gmtxd_are_equal(const gmtxd *a, const gmtxd *b) { return(_gmtx_are_equal(__gmtxp a, __gmtxp b)); }
bool gmtx_are_equal(const gmtx *a, const gmtx *b)    { return(_gmtx_are_equal(a, b)); }

bool gmtxl_is_square(const gmtxl *m) { return(_gmtx_is_square(__gmtxp m)); }
bool gmtxd_is_square(const gmtxd *m) { return(_gmtx_is_square(__gmtxp m)); }
bool gmtx_is_square(const gmtx *m)   { return(_gmtx_is_square(m)); }



/*** VERSION ***/

unsigned int libjmtx_version(void) {

   return(__libj_version);

}
