/* 

   libj - gmtx_rev.c             Copyright (C) 2001-1998 Justin David Smith    
   justins@chaos2.org            http://chaos2.org/
    
   Matrix reversal


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#include <libjmtx.h>
#include <_mtx_int.h>



/*** Matrix Reversal ***/
/* Interface functions */


gmtxl *gmtxl_reverse_horiz(gmtxl *m) {
/* Matrix-Reverse-Horizontally */

   longdata *old;
   longdata *new;
   dimensiona height;
   dimensiona width;
   dimensiona i;
   dimensiona j;

   width = m->width;
   height= m->height;
   i = width / 2;
   while(i) {
      old = MTX_XY_P(m, i, 0);
      new = MTX_XY_P(m, width - i, 0);
      j = height;
      while(j) {
         MTX_SWAP(*old, *new);
         j--;
      }
      i--;
   }
   return(m);
   
}


gmtxd *gmtxd_reverse_horiz(gmtxd *m) {
/* Matrix-Reverse-Horizontally */

   doubledata *old;
   doubledata *new;
   dimensiona height;
   dimensiona width;
   dimensiona i;
   dimensiona j;

   width = m->width;
   height= m->height;
   i = width / 2;
   while(i) {
      old = MTX_XY_P(m, i, 0);
      new = MTX_XY_P(m, width - i, 0);
      j = height;
      while(j) {
         MTX_SWAP(*old, *new);
         j--;
      }
      i--;
   }
   return(m);
   
}


gmtx *gmtx_reverse_horiz(gmtx *m) {
/* Matrix-Reverse-Horizontally */

   if(MTX_IS_LONG(m))   return(__gmtxp gmtxl_reverse_horiz(__gmtxlp m));
   if(MTX_IS_DOUBLE(m)) return(__gmtxp gmtxd_reverse_horiz(__gmtxdp m));
   return(NULL);
   
}


gmtxl *gmtxl_reverse_vert(gmtxl *m) {
/* Matrix-Reverse-Vertically */

   longdata *old;
   longdata *new;
   dimensiona height;
   dimensiona width;
   dimensiona i;
   dimensiona j;

   width = m->width;
   height= m->height;
   j = height / 2;
   old = m->data;
   while(j) {
      new = m->data + width * (height - j);
      i = width;
      while(i) {
         new--;
         MTX_SWAP(*old, *new);
         old++;
         i--;
      }
      j--;
   }
   return(m);
      
}


gmtxd *gmtxd_reverse_vert(gmtxd *m) {
/* Matrix-Reverse-Vertically */

   doubledata *old;
   doubledata *new;
   dimensiona height;
   dimensiona width;
   dimensiona i;
   dimensiona j;

   width = m->width;
   height= m->height;
   j = height / 2;
   old = m->data;
   while(j) {
      new = m->data + width * (height - j);
      i = width;
      while(i) {
         new--;
         MTX_SWAP(*old, *new);
         old++;
         i--;
      }
      j--;
   }
   return(m);
      
}


gmtx *gmtx_reverse_vert(gmtx *m) {
/* Matrix-Reverse-Vertically */

   if(MTX_IS_LONG(m))   return(__gmtxp gmtxl_reverse_vert(__gmtxlp m));
   if(MTX_IS_DOUBLE(m)) return(__gmtxp gmtxd_reverse_vert(__gmtxdp m));
   return(NULL);
   
}


gmtxl *gmtxl_reverse(gmtxl *m) {
/* Matrix-Reverse */

   longdata *old;
   longdata *new;
   dimensiona size;

   size = m->size;
   old = m->data;
   new = m->data + size;
   while(size) {
      new--;
      MTX_SWAP(*old, *new);
      old++;
      size--;
   }
   return(m);
      
}


gmtxd *gmtxd_reverse(gmtxd *m) {
/* Matrix-Reverse */

   doubledata *old;
   doubledata *new;
   dimensiona size;

   size = m->size;
   old = m->data;
   new = m->data + size;
   while(size) {
      new--;
      MTX_SWAP(*old, *new);
      old++;
      size--;
   }
   return(m);
      
}


gmtx *gmtx_reverse(gmtx *m) {
/* Matrix-Reverse */

   if(MTX_IS_LONG(m))   return(__gmtxp gmtxl_reverse(__gmtxlp m));
   if(MTX_IS_DOUBLE(m)) return(__gmtxp gmtxd_reverse(__gmtxdp m));
   return(NULL);
   
}


