/* 

   libj - vmtx.c                 Copyright (C) 2001-1998 Justin David Smith
   justins@chaos2.org            http://chaos2.org/
    
   Basic vector information and comparison


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#include <libjmtx.h>
#include <_mtx_int.h>
#include <libjmem.h>
#include <libjstr.h>



/*** vector information and Comparison ***/
/* Internal functions */


static inline bool _vmtx_is_null(const vmtx *m) {

   /* If width = 0 or height = 0, then vector is NULL */
   return(MTX_IS_SIZE_0(m));

}
   

static inline bool _vmtx_is_vector(const vmtx *m) {

   /* If width = 1 or height = 1, then vector is a valid vector */
   return(MTX_IS_VECTOR(m));
   
}


static inline bool _vmtx_are_equal(const vmtx *a, const vmtx *b) {

   /* Check for NULL matrices first */
   if(_vmtx_is_null(a)) return(_vmtx_is_null(b));
   if(_vmtx_is_null(b)) return(0);
   
   /* Make sure attributes compare */
   if(!MTX_ARE_COMPAT(a, b))  return(0);
   if(!MTX_SAME_SIZE(a, b)) return(0);
   
   /* Compare the raw memory */
   return(jmemeql(a->data, b->data, a->bufsize));

}


static inline bool _vmtx_is_square(const vmtx *m) {

   /* By convention, a NULL vector is square */
   return(MTX_IS_SQUARE(m));
   
}


/*** vector information and Comparison ***/
/* Interface functions */


bool vmtxl_is_null(const vmtxl *m) { return(_vmtx_is_null(__vmtxp m)); }
bool vmtxd_is_null(const vmtxd *m) { return(_vmtx_is_null(__vmtxp m)); }
bool vmtx_is_null(const vmtx *m)   { return(_vmtx_is_null(m)); }

bool vmtxl_is_vector(const vmtxl *m) { return(_vmtx_is_vector(__vmtxp m)); }
bool vmtxd_is_vector(const vmtxd *m) { return(_vmtx_is_vector(__vmtxp m)); }
bool vmtx_is_vector(const vmtx *m)   { return(_vmtx_is_vector(m)); }

bool vmtxl_are_equal(const vmtxl *a, const vmtxl *b) { return(_vmtx_are_equal(__vmtxp a, __vmtxp b)); }
bool vmtxd_are_equal(const vmtxd *a, const vmtxd *b) { return(_vmtx_are_equal(__vmtxp a, __vmtxp b)); }
bool vmtx_are_equal(const vmtx *a, const vmtx *b)    { return(_vmtx_are_equal(a, b)); }

bool vmtxl_is_square(const vmtxl *m) { return(_vmtx_is_square(__vmtxp m)); }
bool vmtxd_is_square(const vmtxd *m) { return(_vmtx_is_square(__vmtxp m)); }
bool vmtx_is_square(const vmtx *m)   { return(_vmtx_is_square(m)); }


