/* 

   libj - vmtx_cons.c            Copyright (C) 2001-1998 Justin David Smith
   justins@chaos2.org            http://chaos2.org/
    
   Vector construction and destruction


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#include <libjmtx.h>
#include <_mtx_int.h>



/*** Constructors/destructors ***/
/* Interface functions */


vmtxl *vmtxl_free(vmtxl **v) {

   return(__vmtxlp gmtxl_free(__gmtxlpp v));
   
}


vmtxd *vmtxd_free(vmtxd **v) {

   return(__vmtxdp gmtxd_free(__gmtxdpp v));
   
}


vmtx *vmtx_free(vmtx **v) {

   return(__vmtxp gmtx_free(__gmtxpp v));
   
}


vmtxl *vmtxl_new(dimensiona size) {

   return(__vmtxlp gmtxl_new(1, size));
   
}


vmtxd *vmtxd_new(dimensiona size) {

   return(__vmtxdp gmtxd_new(1, size));
   
}


vmtx *vmtx_new(uword type, uword dtsize, dimensiona size) {
   
   return(__vmtxp gmtx_new(type, dtsize, 1, size));
   
}


vmtxl *vmtxl_new_using(const vmtxl *v) {

   return(__vmtxlp gmtxl_new_using(__gmtxlp v));
   
}


vmtxd *vmtxd_new_using(const vmtxd *v) {

   return(__vmtxdp gmtxd_new_using(__gmtxdp v));
   
}


vmtx *vmtx_new_using(const vmtx *v) {

   return(__vmtxp gmtx_new_using(__gmtxp v));
   
}


vmtxl *vmtxl_resize(vmtxl *v, dimensiona size) {

   return(__vmtxlp gmtxl_resize(__gmtxlp v, 1, size));
   
}


vmtxd *vmtxd_resize(vmtxd *v, dimensiona size) {

   return(__vmtxdp gmtxd_resize(__gmtxdp v, 1, size));
   
}


vmtx *vmtx_resize(vmtx *v, dimensiona size) {

   return(__vmtxp gmtx_resize(__gmtxp v, 1, size));
   
}


vmtxl *vmtxl_resize_using(vmtxl *vdst, const vmtxl *vsrc) {

   return(__vmtxlp gmtxl_resize_using(__gmtxlp vdst, __gmtxlp vsrc));
   
}


vmtxd *vmtxd_resize_using(vmtxd *vdst, const vmtxd *vsrc) {

   return(__vmtxdp gmtxd_resize_using(__gmtxdp vdst, __gmtxdp vsrc));
   
}


vmtx *vmtx_resize_using(vmtx *vdst, const vmtx *vsrc) {

   return(__vmtxp gmtx_resize_using(__gmtxp vdst, __gmtxp vsrc));
   
}


vmtxl *vmtxl_new_zero(dimensiona size) {

   return(__vmtxlp gmtxl_new_zero(1, size));
   
}


vmtxd *vmtxd_new_zero(dimensiona size) {

   return(__vmtxdp gmtxd_new_zero(1, size));
   
}


vmtxl *vmtxl_new_ones(dimensiona size) {

   return(__vmtxlp gmtxl_new_ones(1, size));
   
}


vmtxd *vmtxd_new_ones(dimensiona size) {

   return(__vmtxdp gmtxd_new_ones(1, size));
   
}


vmtxl *vmtxl_new_const(dimensiona size, longdata value) {

   return(__vmtxlp gmtxl_new_const(1, size, value));
   
}


vmtxd *vmtxd_new_const(dimensiona size, doubledata value) {

   return(__vmtxdp gmtxd_new_const(1, size, value));
   
}


/*** Constructors/Destructors ***/
/* Initialisation functions */


vmtxl *vmtxl_zero(vmtxl *v) {

   return(__vmtxlp gmtxl_zero(__gmtxlp v));
   
}


vmtxd *vmtxd_zero(vmtxd *v) {

   return(__vmtxdp gmtxd_zero(__gmtxdp v));
   
}


vmtx *vmtx_zero(vmtx *v) {

   return(__vmtxp gmtx_zero(__gmtxp v));
   
}


vmtxl *vmtxl_ones(vmtxl *v) {

   return(__vmtxlp gmtxl_ones(__gmtxlp v));
   
}


vmtxd *vmtxd_ones(vmtxd *v) {

   return(__vmtxdp gmtxd_ones(__gmtxdp v));
   
}


vmtx *vmtx_ones(vmtx *v) {

   return(__vmtxp gmtx_ones(__gmtxp v));
   
}


vmtxl *vmtxl_const(vmtxl *v, longdata value) {

   return(__vmtxlp gmtxl_const(__gmtxlp v, value));
   
}


vmtxd *vmtxd_const(vmtxd *v, doubledata value) {

   return(__vmtxdp gmtxd_const(__gmtxdp v, value));
   
}


vmtxl *vmtxl_new_copy(const vmtxl *v) {

   return(__vmtxlp gmtxl_new_copy(__gmtxlp v));

}


vmtxd *vmtxd_new_copy(const vmtxd *v) {

   return(__vmtxdp gmtxd_new_copy(__gmtxdp v));

}


vmtx *vmtx_new_copy(const vmtx *v) {

   return(__vmtxp gmtx_new_copy(__gmtxp v));

}


vmtxl *vmtxl_copy(vmtxl *vdst, const vmtxl *vsrc) {

   return(__vmtxlp gmtx_copy(__gmtxp vdst, __gmtxp vsrc));

}


vmtxd *vmtxd_copy(vmtxd *vdst, const vmtxd *vsrc) {

   return(__vmtxdp gmtx_copy(__gmtxp vdst, __gmtxp vsrc));

}


vmtx *vmtx_copy(vmtx *vdst, const vmtx *vsrc) {

   return(__vmtxp gmtx_copy(__gmtxp vdst, __gmtxp vsrc));

}


