/* 

   libj - vmtx_mult.c            Copyright (C) 2001-1998 Justin David Smith    
   justins@chaos2.org            http://chaos2.org/
    
   Vector multiply


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#include <libjmtx.h>
#include <_mtx_int.h>



/*** Matrix-Vector Multiply ***/
/* Internal code */


static inline vmtx *_vmtx_gmtx_multiply_copy(vmtx *c, const vmtx *a, const gmtx *b) {
/* Vector-Times-Matrix */

   vmtx *vtmp;

   if(MTX_IS_VECTOR_HORIZ(a)) return(__vmtxp gmtx_multiply_copy(__gmtxp c, __gmtxp a, b));
   if(MTX_IS_VECTOR_VERT(a))  {
      if(!(vtmp = __vmtxp gmtx_new_transpose(__gmtxp a))) return(NULL);
      if(!gmtx_multiply_copy(__gmtxp c, __gmtxp a, b)) return(vmtx_free(&vtmp));
      vmtx_free(&vtmp);
      return(c);
   }
   return(NULL);

}


static inline vmtx *_gmtx_vmtx_multiply_copy(vmtx *c, const gmtx *a, const vmtx *b) {
/* Matrix-Times-Vector */

   vmtx *vtmp;

   if(MTX_IS_VECTOR_VERT(a)) return(__vmtxp gmtx_multiply_copy(__gmtxp c, a, __gmtxp b));
   if(MTX_IS_VECTOR_HORIZ(a))  {
      if(!(vtmp = __vmtxp gmtx_new_transpose(__gmtxp b))) return(NULL);
      if(!gmtx_multiply_copy(__gmtxp c, a, __gmtxp b)) return(vmtx_free(&vtmp));
      vmtx_free(&vtmp);
      return(c);
   }
   return(NULL);

}


/*** Matrix/Vector Multiply ***/
/* Interface functions */


vmtxl *vmtxl_gmtxl_multiply_copy(vmtxl *c, const vmtxl *a, const gmtxl *b) {
/* Vector-Times-Matrix */

   return(__vmtxlp _vmtx_gmtx_multiply_copy(__vmtxp c, __vmtxp a, __gmtxp b));
   
}


vmtxd *vmtxd_gmtxd_multiply_copy(vmtxd *c, const vmtxd *a, const gmtxd *b) {
/* Vector-Times-Matrix */

   return(__vmtxdp _vmtx_gmtx_multiply_copy(__vmtxp c, __vmtxp a, __gmtxp b));
   
}


vmtx *vmtx_gmtx_multiply_copy(vmtx *c, const vmtx *a, const gmtx *b) {
/* Vector-Times-Matrix */

   return(_vmtx_gmtx_multiply_copy(c, a, b));
   
}


vmtxl *vmtxl_gmtxl_new_multiply(const vmtxl *a, const gmtxl *b) {
/* Vector-Times-Matrix */

   vmtxl *c;
   
   if(!(c = __vmtxlp vmtx_new_using(__vmtxp a))) return(NULL);
   return(__vmtxlp _vmtx_gmtx_multiply_copy(__vmtxp c, __vmtxp a, __gmtxp b));
   
}


vmtxd *vmtxd_gmtxd_new_multiply(const vmtxd *a, const gmtxd *b) {
/* Vector-Times-Matrix */

   vmtxd *c;
   
   if(!(c = __vmtxdp vmtx_new_using(__vmtxp a))) return(NULL);
   return(__vmtxdp _vmtx_gmtx_multiply_copy(__vmtxp c, __vmtxp a, __gmtxp b));
   
}


vmtx *vmtx_gmtx_new_multiply(const vmtx *a, const gmtx *b) {
/* Vector-Times-Matrix */

   vmtx *c;
   
   if(!(c = vmtx_new_using(a))) return(NULL);
   return(_vmtx_gmtx_multiply_copy(c, a, b));
   
}


vmtxl *gmtxl_vmtxl_multiply_copy(vmtxl *c, const gmtxl *a, const vmtxl *b) {
/* Matrix-Times-Vector */

   return(__vmtxlp _gmtx_vmtx_multiply_copy(__vmtxp c, __gmtxp a, __vmtxp b));
   
}


vmtxd *gmtxd_vmtxd_multiply_copy(vmtxd *c, const gmtxd *a, const vmtxd *b) {
/* Matrix-Times-Vector */

   return(__vmtxdp _gmtx_vmtx_multiply_copy(__vmtxp c, __gmtxp a, __vmtxp b));
   
}


vmtx *gmtx_vmtx_multiply_copy(vmtx *c, const gmtx *a, const vmtx *b) {
/* Matrix-Times-Vector */

   return(_gmtx_vmtx_multiply_copy(c, a, b));
   
}


vmtxl *gmtxl_vmtxl_new_multiply(const gmtxl *a, const vmtxl *b) {
/* Matrix-Times-Vector */

   vmtxl *c;
   
   if(!(c = __vmtxlp vmtx_new_using(__vmtxp b))) return(NULL);
   return(__vmtxlp _gmtx_vmtx_multiply_copy(__vmtxp c, __gmtxp a, __vmtxp b));
   
}


vmtxd *gmtxd_vmtxd_new_multiply(const gmtxd *a, const vmtxd *b) {
/* Matrix-Times-Vector */

   vmtxd *c;
   
   if(!(c = __vmtxdp vmtx_new_using(__vmtxp b))) return(NULL);
   return(__vmtxdp _gmtx_vmtx_multiply_copy(__vmtxp c, __gmtxp a, __vmtxp b));
   
}


vmtx *gmtxvmtx_new_multiply(const gmtx *a, const vmtx *b) {
/* Matrix-Times-Vector */

   vmtx *c;
   
   if(!(c = vmtx_new_using(b))) return(NULL);
   return(_gmtx_vmtx_multiply_copy(c, a, b));
   
}


