/* $Header: /fridge/cvs/libj/jstra/libjstra.h,v 1.4 2001/12/10 19:44:08 justins Exp $ */                                                                                                    
/* 

   libj - jstra.h                Copyright (C) 2001-1998 Justin David Smith
   justins (a) chaos2.org        http://chaos2.org/
    
   Main header file for stringa library


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#ifndef __libj_stra_included
#define __libj_stra_included


/* Definitions and datatypes */
#include <stdio.h>
#include <libj.h>
#include <libjstr.h>


/* StringA implementation */
typedef struct _stringa {
   char *string;        /* Character buffer */
   sizea length;        /* String length */
   sizea bufsize;       /* Buffer size */
} stringa;
#define stra(s)         ((s) != NULL ? (s)->string  :  NULL)
#define strlena(s)      (stra(s) != NULL ? (s)->length  : 0)
#define strsizea(s)     (stra(s) != NULL ? (s)->bufsize : 0)


/* StringA pairs */
typedef struct _stringa_pair {
   stringa *first;
   stringa *second;
} stringa_pair;
#define  paira_car(p)   ((p)->first)
#define  paira_cdr(p)   ((p)->second)


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*** Argument String Processing ***/
stringa *getarga(stringa *d, const char *s, sizea argnum, char sep);
stringa *getargaa(stringa *d, const stringa *s, sizea argnum, char sep);


/*** CGI Functions ***/
bool  getassigna(stringa *var, stringa *val, const char *s);
bool  getassignaa(stringa *var, stringa *val, const stringa *s);


/*** String copy routines ***/
stringa *strcopyna(  stringa *d, const char *s, sizea n);
stringa *strcopynax( stringa *d, const char *s, sizea n);
stringa *strcopynap( stringa *d, const char *s, sizea n, sizea p);
stringa *strcopynaa( stringa *d, const stringa *s, sizea n);
stringa *strcopynaax(stringa *d, const stringa *s, sizea n);
stringa *strcopynaap(stringa *d, const stringa *s, sizea n, sizea p);
stringa *strconcatna(  stringa *d, const char *s, sizea n);
stringa *strconcatnax( stringa *d, const char *s, sizea n);
stringa *strconcatnap( stringa *d, const char *s, sizea n, sizea p);
stringa *strconcatnaa( stringa *d, const stringa *s, sizea n);
stringa *strconcatnaax(stringa *d, const stringa *s, sizea n);
stringa *strconcatnaap(stringa *d, const stringa *s, sizea n, sizea p);
stringa *strcopya(  stringa *d, const char *s);
stringa *strcopyax( stringa *d, const char *s);
stringa *strcopyap( stringa *d, const char *s, sizea p);
stringa *strcopyaa( stringa *d, const stringa *s);
stringa *strcopyaax(stringa *d, const stringa *s);
stringa *strcopyaap(stringa *d, const stringa *s, sizea p);
stringa *strconcata(  stringa *d, const char *s);
stringa *strconcatax( stringa *d, const char *s);
stringa *strconcatap( stringa *d, const char *s, sizea p);
stringa *strconcataa( stringa *d, const stringa *s);
stringa *strconcataax(stringa *d, const stringa *s);
stringa *strconcataap(stringa *d, const stringa *s, sizea p);
stringa *strmovea(stringa *d, stringa *s);
#define  strmoveaa(d, s)   strmovea((d), (s))


/*** String Replacement ***/
sizea    strreplacea(stringa *s, const char *src, const char *dst);
sizea    strreplaceaa(stringa *s, const stringa *src, const stringa *dst);
sizea    strreplacea_list(stringa *s, const string_pair *rewrites);
sizea    strreplaceaa_list(stringa *s, const stringa_pair *rewrites);
stringa *strdeletea(stringa *s, sizea offset, sizea length);
stringa *strtrunca(stringa *s, sizea n);
stringa *strinserta(stringa *s, sizea offset, const char *ins);
stringa *strinsertaa(stringa *s, sizea offset, const stringa *ins);
stringa *strdelinsa(stringa *s, sizea offset, sizea length, const char *ins);
stringa *strdelinsaa(stringa *s, sizea offset, sizea length, const stringa *ins);


/*** String Printf-style Functions ***/
stringa *sprintfa_offset(stringa *d, sizea offset, const char *fmt, ...);  /* Don't use me! */
#define  sprintfa(d, fmt...)           sprintfa_offset((d), 0, fmt)
#define  sprintfa_concat(d, fmt...)    sprintfa_offset((d), strlena(d), fmt)


/*** Comparisons ***/
bool     strequalaa(const stringa *a, const stringa *b);
bool     strequalnaa(const stringa *a, const stringa *b, sizea n);


/*** Special string creation routines ***/
stringa *strnewa(const char *s);
stringa *strclonea(const stringa *s);
#define  strnewaa(s)    strclonea((s))
void strfreea(stringa *s);
void strreleasea(stringa **s);
void strcollapsea(stringa **s);


/*** String control (lowlevel) ***/
/* These utilities do not currently appear to be defined */
#ifdef __undefined
sizea     strminsizea(stringa *d, sizea s);     /* Set buffer minimum size to <s> bytes */
sizea     strtrunca(stringa *d, sizea s);       /* Truncate the string to <s> bytes     */
sizea     strsetlena(stringa *d);               /* Recalculate the length of the string */
#endif


/*** File I/O ***/
bool freada(stringa *d, FILE *f);


#ifdef __cplusplus /* Close the extern */
};
#endif /* __cplusplus */


/* Miscellaneous macros (requires libjstr) */
#define strcompa(a, b)           strcomp(stra(a), (b))
#define strcompaa(a, b)          strcomp(stra(a), stra(b))
#define strcompna(a, b, n)       strcompn(stra(a), (b), (n))
#define strcompnaa(a, b, n)      strcompn(stra(a), stra(b), (n))
#define strequala(a, b)          strequal(stra(a), (b))
#define strequalna(a, b, n)      strequaln(stra(a), (b), (n))
#define getargpa(a, b, c, d)     getargp((a), stra(b), (c), (d))
#define getargna(a, b, c, d, n)  getargn((a), stra(b), (c), (d), (n))
#define getnumargsa(a, b)        getnumargs(stra(a), (b))
#define putslenna(a, n)          putslenn(stra(a), (n))
#define unescapeCGIa(s)          unescapeCGI(stra(s))
#define trima(s)                 trim(stra(s))
#define trimcommenta(s, c)       trimcomment(stra(s), (c))
#define strtointa(s, a)          strtoint(stra(s), (a))
#define strscana(s, pat)         strscan(stra(s), (pat))
#define strscanaa(s, pat)        strscan(stra(s), stra(pat))


#endif /* Header included */
