/* $Header: /fridge/cvs/libj/jstra/stra_copy.c,v 1.3 2001/12/10 19:44:08 justins Exp $ */                                                                                                    
/* 

   libj - stra_copy.c            Copyright (C) 2001-1998 Justin David Smith
   justins (a) chaos2.org        http://chaos2.org/
    
   StringA copy/concatenate functions


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#define  ALLOW_LIBC_STRING    1
#include <libjstra.h>
#include <_str.h>


#define  __need_newminsizea   1
#define  __need_minsizea      1
#define  __need_freea         1
#define  __need_cpya          1
#define  __need_cpyaa         1
#define  __need_cata          1
#define  __need_cataa         1
#include <_stra_alloc.h>



stringa *strcopya(stringa *d, const char *s) {
/* String-Copy-A */

   if(d != NULL) return(__strcpya(d, s));   
   return(NULL);

}



stringa *strcopyaa(stringa *d, const stringa *s) {
/* String-Copy-A-To-A */

   if(d != NULL) return(__strcpyaa(d, s));   
   return(NULL);

}



stringa *strcopyax(stringa *d, const char *s) {
/* String-Copy-A-Nofree
   This is similar to strcopya, except this function will NOT attempt to
   free the current contents of *d. Note that new memory will still be
   allocated for the string and as a result the contents at the old 
   pointer will be preserved. However, the pointer itself will change so
   make sure you have saved the old pointer elsewhere before calling 
   this. */

   sizea slen;

   if(d != NULL) {
      d->string = NULL;

      if(s != NULL) {
         /* Find length of source. */
         slen = STRLEN(s);

         /* Create new string buffer if needed */
         __strnewminsizea(d, slen);

         /* Copy the string */
         d->length = slen;
         if(d->string != NULL) MEMCPY(d->string, s, slen + 1);
      }
   }

   return(d);

}



stringa *strcopyaax(stringa *d, const stringa *s) {
/* String-Copy-A-To-A-Nofree
   Can assume s is a StringA string. */

   sizea slen;

   if(d != NULL) {
      d->string = NULL;

      if(s != NULL && s->string != NULL) {
         /* Find length of source. */
         slen = s->length;

         /* Create new string buffer if needed */
         __strnewminsizea(d, slen);

         /* Copy the string */
         d->length = slen;
         if(d->string != NULL) MEMCPY(d->string, s->string, slen + 1);
      } 
   }

   return(d);

}



stringa *strcopyap(stringa *d, const char *s, sizea size) {
/* String-Copy-A-Padding
   This is similar to strcopya except <size> padding bytes will be
   introduced after the NULL character for the <*d> string. Pad
   out your buffers, fool! I'm sure they're just waiting for a
   buffer overflow when you call a non-stringA function! */

   sizea slen;

   if(d != NULL) {
      /* Find length of source. */
      slen = STRLENN(s);

      /* Create new string buffer if needed */
      __strnewminsizea(d, slen + size);

      /* Copy the string */
      d->length = slen;
      if(d->string != NULL) {
         if(s != NULL) MEMCPY(d->string, s, slen + 1);
         else *d->string = '\0';
      }
   }

   return(d);

}



stringa *strcopyaap(stringa *d, const stringa *s, sizea size) {
/* String-Copy-A-To-A-Padding
   Same as above, but assume source string is a StringA. */

   sizea slen;

   if(d != NULL) {
      /* Find length of source. */
      slen = strlena(s);

      /* Create new string buffer if needed */
      __strnewminsizea(d, slen + size);

      /* Copy the string */
      d->length = slen;
      
      if(d->string != NULL) {
         if(s != NULL && s->string != NULL) MEMCPY(d->string, s->string, slen + 1);
         else *d->string = '\0';
      }
   }

   return(d);

}



stringa *strconcata(stringa *d, const char *s) {
/* String-Cat-A */

   return(__strcata(d, s));   

}



stringa *strconcataa(stringa *d, const stringa *s) {
/* String-Cat-A-To-A */

   return(__strcataa(d, s));   

}



stringa *strconcatax(stringa *d, const char *s) {
/* String-Cat-A-Nofree 
   Similar to strconcata, except the old <*d> buffer will
   not be freed. Instead, a new buffer will be created, and both
   <*d> and <s> will be copied into the new buffer. 

   Warning: The length of *d must be properly set (see strsetlena()) */

   char *oldstring;

   if(d != NULL) {
      oldstring = d->string;
      d->string = NULL;
      __strcpya(d, oldstring);
      __strcata(d, s);
   }
   return(d);

}



stringa *strconcataax(stringa *d, const stringa *s) {
/* String-Cat-A-To-A-Nofree 
   Similar to strconcata, except the old <*d> buffer will
   not be freed. Instead, a new buffer will be created, and both
   <*d> and <s> will be copied into the new buffer. 

   Warning: The length of *d must be properly set (see strsetlena()) */

   stringa oldstring;
   
   if(d != NULL) {
      oldstring.string = d->string;
      oldstring.length = d->length;
      d->string = NULL;
      __strcpyaa(d, &oldstring);
      __strcataa(d, s);
   }
   return(d);

}



stringa *strconcatap(stringa *d, const char *s, sizea size) {
/* String-Cat-A-Padding
   Similar to strconcata, with the effect of strcopyap. */

   sizea slen;
   sizea dlen;

   if(d != NULL) {
      dlen = strlena(d);
      slen = STRLENN(s);

      /* Create the new string buffer */
      __strminsizea(d, slen + dlen + size);

      /* Concatenate the strings on in. */
      d->length = slen + dlen;
      if(d->string != NULL) {
         if(s != NULL) MEMCPY(d->string + dlen, s, slen + 1);
         else *(d->string + dlen) = '\0';
      }
   }
   return(d);

}



stringa *strconcataap(stringa *d, const stringa *s, sizea size) {
/* String-Cat-A-To-A-Padding
   Similar to strconcataa, with the effect of strcopyaap. */

   sizea slen;
   sizea dlen;

   if(d != NULL) {
      dlen = strlena(d);
      slen = strlena(s);

      /* Create the new string buffer */
      __strminsizea(d, slen + dlen + size);

      /* Concatenate the strings on in. */
      d->length = slen + dlen;
      if(d->string != NULL) {
         if(s != NULL) MEMCPY(d->string + dlen, s, slen + 1);
         else *(d->string + dlen) = '\0';
      }
   }
   return(d);

}


stringa *strcopyna(stringa *d, const char *s, sizea n) {
/* String-N-Copy-A
   Another StringA function, yup yup yup. This one copies a maximum of
   <n> characters from <s> into a target string <*d>. Other than that,
   this is just another really boring function. Keep an eye out for 
   the StringB type which will come along Real Soon Now (tm). */

   sizea slen;

   if(d != NULL) {
      if(s != NULL) {
         slen = STRLEN(s);
         if(slen < n) n = slen;
         __strnewminsizea(d, n + 1);
         if(d->string != NULL) {
            MEMCPY(d->string, s, n);
            *(d->string + n) = '\0';
         }
         d->length = n;
      } else {
         __strfreea(d);
      }
   }

   return(d);
   
}



stringa *strcopynaa(stringa *d, const stringa *s, sizea n) {
/* String-N-Copy-A-To-A
   Another StringA function, yup yup yup. This one copies a maximum of
   <n> characters from <s> into a target string <*d>. Other than that,
   this is just another really boring function. Keep an eye out for 
   the StringB type which will come along Real Soon Now (tm). 
   
   This function assumes s is a StringA string. */

   sizea slen;

   if(d != NULL) {
      if(s != NULL) {
         slen = s->length;
         if(slen < n) n = slen;
         __strnewminsizea(d, n + 1);
         if(d->string != NULL) {
            MEMCPY(d->string, s->string, n);
            *(d->string + n) = '\0';
         }
         d->length = n;
      } else {
         __strfreea(d);
      }
   }

   return(d);
   
}



stringa *strmovea(stringa *d, stringa *s) {
/* String-Move-Pointer
   A StringA function which moves the string pointed to by <*s> into the
   location <*d>, sets <*s> to NULL and releases any pre-existing strings
   in <*d>. It's faster than string copy between two StringA-type strings. */

   if(s != NULL && d != NULL) {
      FREE(d->string);
      d->string = s->string;
      d->length = s->length;
      d->bufsize = s->bufsize;
      s->string = NULL;
      s->length = 0;
      s->bufsize = 0;
      return(d);
   }
   return(NULL);
   
}


