/* $Header: /fridge/cvs/libj/jstra/stra_printf.c,v 1.2 2001/12/10 19:44:08 justins Exp $ */                                                                                                    
/*
  
   libj - stra_printf.c          Copyright (C) 2001 Justin David Smith
   justins (a) chaos2.org        http://chaos2.org/

   sprintfa helper functions


   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program, in the file COPYING. If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA


   This file is part of LIBJ.

*/
#include <libjstra.h>
#include <stdarg.h>


#define  __need_newminsizea   1
#include <_stra_alloc.h>



stringa *sprintfa_offset(stringa *dest, sizea offset, const char *fmt, ...) {
/* sprintfa
   Calls sprintf to print some data into the stringa given in dest.  Note
   that this function isn't particularly efficient; also, we will start 
   the write at offset; offset must be smaller than the current bufsize.  */

   va_list args;     /* Variable-argument list */
   int nchars;       /* Number of chars required, or -1: fail */
   int size;         /* Size of buffer (or requested size) */
   
   /* Make sure dest and fmt are valid, at least. */
   if(dest == NULL || fmt == NULL) return(dest);
   
   /* Initial size is the buffer size for dest. */
   size = strsizea(dest);
   
   /* Loop until we succeed at printing the string */
   do {
      /* Attempt to write the string */
      va_start(args, fmt);
      nchars = vsnprintf(stra(dest) + offset, size - offset, fmt, args);
      va_end(args);
      
      /* Verification.  CAUTION CAUTION CAUTION
         Old glibc returns -1 on fail, newer glibc
         return the number of characters required
         to complete the string! */
      if(nchars > -1 && nchars < size) {
         /* Success!  Don't forget to set the new length */
         dest->length = offset + nchars;
         return(dest);
      } else if(nchars > -1) {
         /* Try allocating nchars+1 */
         size = offset + nchars + 1;
      } else {
         /* Unknown request: try doubling */
         size = offset + size * 2;
      }
      
      /* Resize the output buffer */
      __strnewminsizea(dest, size);
   } while(true);

}



