/* $Header: /fridge/cvs/libj/jstra/stra_replace.c,v 1.4 2001/12/10 19:44:08 justins Exp $ */                                                                                                    
/* 

   libj - str_replace.c          Copyright (C) 2001 Justin David Smith
   justins (a) chaos2.org        http://chaos2.org/
    
   String Replacement functions


   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program, in the file COPYING. If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
   
   
   This file is part of LIBJ.
   
*/
#define  ALLOW_LIBC_STRING    1
#include <libjstra.h>
#include <_str.h>


#define  __need_newminsizea   1
#define  __need_freea         1
#define  __need_minsizea      1
#define  __need_cpya          1
#define  __need_cpyaa         1
#define  __need_cata          1
#define  __need_cataa         1
#define  __need_trunc         1
#include <_stra_alloc.h>



static inline sizea __strreplacea(stringa *in_s, const char *src, const char *dst, sizea srclen, sizea dstlen) {
/* strreplacea
   Replaces occurrences of src with dst in the string s.  We guarantee that 
   s will terminate with NULL after this call; we may be forced to grow the 
   buffer s during this call.  If src or s are NULL, nothing happens; if dst 
   is NULL, then occurrences of src are removed.  NOTE: we do not scan dst 
   for further replacements in this function.  Note that srclen and dstlen
   specify the lengths of the src and dst strings. */

   char *s;          /* Pointer past last successful replace */
   char *p;          /* Pointer to beginning of first match */
   sizea p_ofs;      /* Offset for p (used when we must grow) */
   sizea slen;       /* Number of chars remaining in s, to care about */
   sizea ssize;      /* Current allocation request for s */
   sizea count = 0;  /* Number of replacements done */

   /* Sanity checks */
   if(in_s == NULL || src == NULL || *src == '\0') return(0);
   
   /* Prepare to search for occurrences of src */
   s = stra(in_s);
   slen = strlena(in_s);
   ssize = strlena(in_s);

   while((p = strscan(s, src)) != NULL) {
      /* Found an occurrence of src */
      ++count;

      /* We can now ignore characters before p, and will also ignore the
         characters in the src string that we just matched. */
      slen -= (p - s) + srclen;
      in_s->length += dstlen - srclen;

      /* What needs to shift where? */
      if(dst == NULL) {
         /* Delete the source string */
         MEMMOVE(p, p + srclen, slen + 1);
         s = p;
      } else if(dstlen <= srclen) {
         /* Tail of s will be moved forward a bit. */
         s = p + dstlen;
         if(dstlen != srclen) MEMMOVE(s, p + srclen, slen + 1);
         MEMCPY(p, dst, dstlen);
      } else {
         /* The stringa buffer may have to grow in this case. */
         ssize += dstlen - srclen;
         p_ofs = p - stra(in_s);
         __strminsizea(in_s, ssize);
         
         /* Resize was hopefully successful.  Restore p now */
         p = stra(in_s) + p_ofs;    /* p points to the replace spot */
         s = p + dstlen;            /* s points to where remainder of string will go */
         MEMMOVE(s, p + srclen, slen + 1);
         MEMCPY(p, dst, dstlen);
      } /* All possible replacement cases */
   } /* Found a string to rewrite */

   /* Return the number of replacements that happened */
   return(count);

}



sizea strreplacea(stringa *in_s, const char *src, const char *dst) {
/* strreplacea */

   return(__strreplacea(in_s, src, dst, STRLENN(src), STRLENN(dst)));

}



sizea strreplaceaa(stringa *in_s, const stringa *src, const stringa *dst) {
/* strreplaceaa */

   return(__strreplacea(in_s, stra(src), stra(dst), strlena(src), strlena(dst)));

}



static inline sizea __strreplacea_list(stringa *in_s, const char *const*src, const char *const*dst, const sizea *srclen, const sizea *dstlen) {
/* strreplacea_list
   Replaces occurrences of src with dst in the string s, where (src, dst)
   is any pair in the list repl that is passed in.  The list should end
   with a (NULL, NULL) pair to signal the end of the list (actually, when
   src == NULL we will always terminate).  The sizes of src and dst should
   be conveniently encoded in (srclen, dstlen).  */

   char *s;          /* Pointer past last successful replace */
   char *p;          /* Pointer to beginning of first match */
   sizea p_ofs;      /* Offset for p (used when we must grow) */
   sizea slen;       /* Number of chars remaining in s, to care about */
   sizea ssize;      /* Current allocation request for s */
   sizea index;      /* The index into src identifying the match */
   sizea i_srclen;   /* Length corresponding to index'th src element */
   sizea i_dstlen;   /* Length corresponding to index'th dst element */
   sizea count = 0;  /* Number of replacements done */

   /* Sanity checks */
   if(in_s == NULL || src == NULL || dst == NULL || srclen == NULL || dstlen == NULL) return(0);
   
   /* Prepare to search for occurrences of src */
   s = stra(in_s);
   slen = strlena(in_s);
   ssize = strlena(in_s);

   while((p = strscan_list(s, src, &index)) != NULL) {
      /* Found an occurrence of src */
      ++count;
      i_srclen = srclen[index];
      i_dstlen = dstlen[index];

      /* We can now ignore characters before p, and will also ignore the
         characters in the src string that we just matched. */
      slen -= (p - s) + i_srclen;
      in_s->length += i_dstlen - i_srclen;

      /* What needs to shift where? */
      if(i_dstlen <= 0) {
         /* Delete the source string */
         MEMMOVE(p, p + i_srclen, slen + 1);
         s = p;
      } else if(i_dstlen <= i_srclen) {
         /* Tail of s will be moved forward a bit. */
         s = p + i_dstlen;
         if(i_dstlen != i_srclen) MEMMOVE(s, p + i_srclen, slen + 1);
         MEMCPY(p, dst[index], i_dstlen);
      } else {
         /* The stringa buffer may have to grow in this case. */
         ssize += i_dstlen - i_srclen;
         p_ofs = p - stra(in_s);
         __strminsizea(in_s, ssize);
         
         /* Resize was hopefully successful.  Restore p now */
         p = stra(in_s) + p_ofs;    /* p points to the replace spot */
         s = p + i_dstlen;          /* s points to where remainder of string will go */
         MEMMOVE(s, p + i_srclen, slen + 1);
         MEMCPY(p, dst[index], i_dstlen);
      } /* All possible replacement cases */
   } /* Found a string to rewrite */

   /* Return the number of replacements that happened */
   return(count);

}



sizea strreplacea_list(stringa *s, const string_pair *repl) {
/* strreplacea_list
   Applies all replacements in the indicated list in parallel.  The list should
   be terminated with an entry that has the first pointer set to NULL.  In any
   other entry, the first pointer corresponds to src (text to be replaced) and 
   the second pointer will correspond to dst (replacement text) */

   char **src;
   char **dst;
   sizea *srclen;
   sizea *dstlen;
   const string_pair *prepl;
   char **psrc;
   char **pdst;
   sizea *psrclen;
   sizea *pdstlen;
   sizea numentries;
   sizea i;

   /* Determine number of entries */
   prepl = repl;
   while(prepl->first != NULL && *(prepl->first) != '\0') ++prepl;
   numentries = prepl - repl + 1;

   /* Allocate temporary structures */
   src = (char **)malloc(sizeof(char *) * numentries);
   dst = (char **)malloc(sizeof(char *) * numentries);
   srclen = (sizea *)malloc(sizeof(sizea) * numentries);
   dstlen = (sizea *)malloc(sizeof(sizea) * numentries);
   if(src == NULL || dst == NULL || srclen == NULL || dstlen == NULL) return(-1);
   
   /* Initialise temporary structures */
   prepl = repl;
   psrc = src;
   pdst = dst;
   psrclen = srclen;
   pdstlen = dstlen;
   for(i = 0; i < numentries - 1; ++prepl, ++psrc, ++pdst, ++psrclen, ++pdstlen, ++i) {
      *psrc = prepl->first;
      *pdst = prepl->second;
      *psrclen = STRLENN(*psrc);
      *pdstlen = STRLENN(*pdst);
   }
   *psrc = NULL;
   *pdst = NULL;

   /* Call the replace engine */
   i = __strreplacea_list(s, (const char *const*)src, (const char *const*)dst, srclen, dstlen);
   
   /* Release temporary structures */
   free(src);
   free(dst);
   free(srclen);
   free(dstlen);
   
   /* Return result */
   return(i);
   
}



stringa *strdeletea(stringa *s, sizea offset, sizea length) {
/* strdeletea
   Deletes a substring from s, starting at offset and with the length
   indicated.  If offset + length exceeds the size of the string, then
   the string is truncated.  */
   
   sizea    tailsize;      /* #chars in tail to be moved */

   /* Check that offset is legal and length is positive */
   if(offset >= strlena(s) || length <= 0) return(s);
   
   /* Check whether the entire tail is truncated? */
   if(offset + length >= strlena(s)) {
      /* Truncation is easy */
      s->string[offset] = '\0';
      s->length = offset;
      return(s);
   }
   
   /* Darn.  We have to actually move some data back. */
   tailsize = strlena(s) - (offset + length);
   MEMMOVE(s->string + offset, s->string + offset + length, tailsize + 1);
   s->length -= length;
   return(s);
   
}



stringa *strtrunca(stringa *s, sizea n) {
/* strtrunca
   Truncates a string so it only contains the first n characters. */
   
   return(strdeletea(s, n, strlena(s)));

}



stringa *strinserta(stringa *s, sizea offset, const char *ins) {
/* strinserta
   Inserts the string ins into s, starting at the offset given.  If offset
   is past the end of s, then ins will be appended to the string (much like
   concat).  */

   sizea    inslen;        /* #chars in string ins */   
   sizea    tailsize;      /* #chars in tail to be moved */
   sizea    ssize;         /* New allocation request */
   
   /* Check that ins contains anything */
   if(ins == NULL || *ins == '\0') return(s);
   
   /* If offset is past end of string, use concat */
   if(offset >= strlena(s)) return(__strcata(s, ins));

   /* We require more space; send a new allocation request. */
   inslen = STRLEN(ins);
   tailsize = strlena(s) - offset;
   ssize = strsizea(s) + inslen;
   __strminsizea(s, ssize);

   /* Move part of the tail of the string to make space */
   MEMMOVE(s->string + offset + inslen, s->string + offset, tailsize + 1);
   MEMCPY(s->string + offset, ins, inslen);
   s->length += inslen;
   return(s);

}



stringa *strinsertaa(stringa *s, sizea offset, const stringa *ins) {
/* strinsertaa
   Inserts the string ins into s, starting at the offset given.  If offset
   is past the end of s, then ins will be appended to the string (much like
   concat).  */

   sizea    tailsize;      /* #chars in tail to be moved */
   sizea    ssize;         /* New allocation request */
   
   /* Check that ins contains anything */
   if(stra(ins) == NULL || *stra(ins) == '\0') return(s);
   
   /* If offset is past end of string, use concat */
   if(offset >= strlena(s)) return(__strcataa(s, ins));

   /* We require more space; send a new allocation request. */
   tailsize = strlena(s) - offset;
   ssize = strsizea(s) + strlena(ins);
   __strminsizea(s, ssize);

   /* Move part of the tail of the string to make space */
   MEMMOVE(s->string + offset + strlena(ins), s->string + offset, tailsize + 1);
   MEMCPY(s->string + offset, stra(ins), strlena(ins));
   s->length += strlena(ins);
   return(s);

}



stringa *strdelinsa(stringa *s, sizea offset, sizea length, const char *ins) {
/* strdelinsa */

   strdeletea(s, offset, length);
   strinserta(s, offset, ins);
   return(s);
   
}



stringa *strdelinsaa(stringa *s, sizea offset, sizea length, const stringa *ins) {
/* strdelinsaa */

   strdeletea(s, offset, length);
   strinsertaa(s, offset, ins);
   return(s);
   
}



