/* $Header: /fridge/cvs/libj/jstra/stra_test.c,v 1.2 2001/12/10 19:44:08 justins Exp $ */                                                                                                    
/* 

   libj - stra_test.c            Copyright (C) 2001 Justin David Smith
   justins (a) chaos2.org        http://chaos2.org/
    
   Tests the jstra functions


   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this library; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
   
   This file is part of LIBJ.
   
*/
#define  ALLOW_LIBC_STRING    1
#include <libjstra.h>



int   success  =  0;
int   failure  =  0;



void test_int(const char *msg, int actual, int expected) {

   if(actual == expected) {
      printf("%16s: pass\n", msg);
      ++success;
   } else {
      printf("%16s: fail - expected %d, got %d\n", msg, expected, actual);
      ++failure;
   }

}



void test_int_gt(const char *msg, int actual, int expected) {

   if(actual > expected) {
      printf("%16s: pass (%d > %d)\n", msg, actual, expected);
      ++success;
   } else {
      printf("%16s: fail - %d !> %d\n", msg, actual, expected);
      ++failure;
   }

}



void test_ptr(const char *msg, const void *actual, const void *expected) {

   if(actual == expected) {
      printf("%16s: pass\n", msg);
      ++success;
   } else {
      printf("%16s: fail - expected %p, got %p\n", msg, expected, actual);
      ++failure;
   }

}



void test_str(const char *msg, const char *actual, const char *expected) {

   if(strcmp(actual, expected) == 0) {
      printf("%16s: pass\n", msg);
      ++success;
   } else {
      printf("%16s: fail - buffers differ, expected \"%s\", got \"%s\"\n", msg, expected, actual);
      ++failure;
   }

}




int main(int argc, char **argv) {

   stringa *s1;
   stringa *s2;
   stringa *s31;
   stringa *s32;
   stringa *s33;
   char moogle[0x1000];
   int i;
   
   for(i = 0; i < 32; ++i) {
      memcpy(moogle + i * 8, "moogle  ", 8);
   }
   moogle[i * 8] = '\0';

   /* Test initialisation and whatnot */
   s1 = NULL;
   test_ptr("stra0",          stra(s1), NULL);
   test_int("strlena0",       strlena(s1), 0);
   test_int("strsizea0",      strsizea(s1), 0);
   s1 = strnewa(NULL);
   test_int("strnewa1a",      strlena(s1), 0);
   test_int_gt("strnewa1b",   strsizea(s1), 0);
   test_str("strnewa1c",      stra(s1), "");
   strreleasea(&s1);
   test_ptr("strreleasea1",   s1, NULL);
   strreleasea(&s1);
   test_ptr("strreleasea2",   s1, NULL);
   s1 = strnewa(moogle);
   test_int("strnewa2a",      strlena(s1), 256);
   test_int_gt("strnewa2b",   strsizea(s1), 256);
   test_str("strnewa2c",      stra(s1), moogle);
   s2 = strclonea(NULL);
   test_int("strclonea1a",    strlena(s2), 0);
   test_int("strclonea1b",    strsizea(s2), 0);
   test_ptr("strclonea1c",    stra(s2), NULL);
   strreleasea(&s2);
   s2 = strclonea(s1);
   test_int("strclonea2a",    strlena(s2), 256);
   test_int_gt("strclonea2b", strsizea(s2), 256);
   test_str("strclonea2c",    stra(s2), moogle);
   strreleasea(&s2);
   strreleasea(&s1);

   /* Test copy functions */
   s1 = strnewa(moogle);
   s2 = strnewa(NULL);
   test_ptr("strcopya1a",     strcopya(s2, stra(s1)), s2);
   test_str("strcopya1b",     stra(s2), moogle);
   test_int("strcopya1c",     strlena(s2), 256);
   test_ptr("strcopya2a",     strcopya(s2, NULL), s2);
   test_ptr("strcopya2b",     stra(s2), NULL);
   test_ptr("strcopya3",      strcopya(NULL, stra(s1)), NULL);
   test_ptr("strcopyaa1a",    strcopyaa(s2, s1), s2);
   test_str("strcopyaa1b",    stra(s2), moogle);
   test_int("strcopyaa1c",    strlena(s2), 256);
   test_ptr("strcopyaa2a",    strcopyaa(s2, NULL), s2);
   test_ptr("strcopyaa2b",    stra(s2), NULL);
   test_ptr("strcopyaa3",     strcopyaa(NULL, s1), NULL);
   test_ptr("strcopyna1a",    strcopyna(s2, stra(s1), 16), s2);
   test_str("strcopyna1b",    stra(s2), "moogle  moogle  ");
   test_int("strcopyna1c",    strlena(s2), 16);
   test_ptr("strcopyna2a",    strcopyna(s2, stra(s1), 512), s2);
   test_str("strcopyna2b",    stra(s2), moogle);
   test_ptr("strcopyna3a",    strcopyna(s2, NULL, 512), s2);
   test_ptr("strcopyna3b",    stra(s2), NULL);
   test_ptr("strcopyna4",     strcopyna(NULL, stra(s1), 512), NULL);
   test_ptr("strcopyna5a",    strcopyna(s2, stra(s1), 0), s2);
   test_str("strcopyna5b",    stra(s2), "");
   test_ptr("strcopynaa1a",   strcopynaa(s2, s1, 16), s2);
   test_str("strcopynaa1b",   stra(s2), "moogle  moogle  ");
   test_int("strcopynaa1c",   strlena(s2), 16);
   test_ptr("strcopynaa2a",   strcopynaa(s2, s1, 512), s2);
   test_str("strcopynaa2b",   stra(s2), moogle);
   test_ptr("strcopynaa3a",   strcopynaa(s2, NULL, 512), s2);
   test_ptr("strcopynaa3b",   stra(s2), NULL);
   test_ptr("strcopynaa4",    strcopynaa(NULL, s1, 512), NULL);
   test_ptr("strcopynaa5a",   strcopynaa(s2, s1, 0), s2);
   test_str("strcopynaa5b",   stra(s2), "");

   /* Concatenation had damn well better work */
   #define  MOOGLE1  "moogle!#"
   #define  MOOGLE2  "kupomog!"
   #define  MOOGLE3  "[[ozma]]"
   test_ptr("strconcata1a",   strcopya(s2, MOOGLE1 MOOGLE3), s2);
   test_ptr("strconcata1b",   strconcata(s2, MOOGLE2), s2);
   test_str("strconcata1c",   stra(s2), MOOGLE1 MOOGLE3 MOOGLE2);
   test_int("strconcata1d",   strlena(s2), 24);
   test_ptr("strconcata2a",   strcopya(s2, NULL), s2);
   test_ptr("strconcata2b",   strconcata(s2, MOOGLE3), s2);
   test_str("strconcata2c",   stra(s2), MOOGLE3);
   test_int("strconcata2d",   strlena(s2), 8);
   test_ptr("strconcata3a",   strcopya(s2, MOOGLE1 MOOGLE3), s2);
   test_ptr("strconcata3b",   strconcata(s2, NULL), s2);
   test_str("strconcata3c",   stra(s2), MOOGLE1 MOOGLE3);
   test_int("strconcata3d",   strlena(s2), 16);
   test_ptr("strconcata4",    strconcata(NULL, stra(s2)), NULL);
   s31 = strnewa(MOOGLE1);
   s32 = strnewa(MOOGLE2);
   s33 = strnewa(MOOGLE3);
   test_ptr("strconcataa1a",  strcopyaa(s2, s31), s2);
   test_ptr("strconcataa1b",  strconcataa(s2, s32), s2);
   test_str("strconcataa1c",  stra(s2), MOOGLE1 MOOGLE2);
   test_int("strconcataa1d",  strlena(s2), 16);
   test_ptr("strconcataa2a",  strcopyaa(s2, NULL), s2);
   test_ptr("strconcataa2b",  strconcataa(s2, s33), s2);
   test_str("strconcataa2c",  stra(s2), MOOGLE3);
   test_int("strconcataa2d",  strlena(s2), 8);
   test_ptr("strconcataa3a",  strcopyaa(s2, s32), s2);
   test_ptr("strconcataa3b",  strconcataa(s2, NULL), s2);
   test_str("strconcataa3c",  stra(s2), MOOGLE2);
   test_int("strconcataa3d",  strlena(s2), 8);
   test_ptr("strconcataa4",   strconcataa(NULL, s2), NULL);
   strreleasea(&s31);
   strreleasea(&s32);
   strreleasea(&s33);

   /* Resize test */
   test_ptr("resize1",        strcopyaa(s2, NULL), s2);
   test_int("resize2",        strsizea(s2), 0);
   test_ptr("resize3",        strconcataa(s2, s1), s2);
   test_int_gt("resize4",     strsizea(s2), 256);
   test_ptr("resize5",        strconcataa(s2, s1), s2);
   test_int_gt("resize6",     strsizea(s2), 512);
   test_ptr("resize7",        strconcataa(s2, s1), s2);
   test_int_gt("resize8",     strsizea(s2), 768);
   test_ptr("resize9",        strconcataa(s2, s1), s2);
   test_int_gt("resize10",    strsizea(s2), 1024);
   test_int("resize11",       strlena(s2), 1024);
   test_str("resize12",       stra(s2) + 768, moogle);

   /* Test strmove */
   s31 = strnewa(NULL);
   strmovea(s31, s1);
   test_ptr("strmove1",       stra(s1), NULL);
   test_int("strmove2",       strsizea(s1), 0);
   test_int("strmove3",       strlena(s1), 0);
   test_str("strmove4",       stra(s31), moogle);
   test_int_gt("strmove5",    strsizea(s31), 256);
   test_int("strmove6",       strlena(s31), 256);
   strmovea(s1, s31);
   test_ptr("strmove7",       stra(s31), NULL);
   test_int("strmove8",       strsizea(s31), 0);
   test_int("strmove9",       strlena(s31), 0);
   test_str("strmove10",      stra(s1), moogle);
   test_int_gt("strmove11",   strsizea(s1), 256);
   test_int("strmove12",      strlena(s1), 256);
   strreleasea(&s31);

   /* Test strreplacea */
   strcopya(s1, "abcab$replcabca$replbc");
   test_int("strreplacea1a",     strreplacea(s1, "$repl", NULL), 2);
   test_str("strreplacea1b",     stra(s1), "abcabcabcabc");
   test_int_gt("strreplacea1c",  strsizea(s1), 12);
   test_int("strreplacea1d",     strlena(s1), 12);
   strcopya(s1, "abcab$replcabca$replbc");
   test_int("strreplacea2a",     strreplacea(s1, "$repl", "X"), 2);
   test_str("strreplacea2b",     stra(s1), "abcabXcabcaXbc");
   test_int_gt("strreplacea2c",  strsizea(s1), 14);
   test_int("strreplacea2d",     strlena(s1), 14);
   strcopya(s1, "abcab$replcabca$replbc");
   test_int("strreplacea3a",     strreplacea(s1, "$repl", "XXXXX"), 2);
   test_str("strreplacea3b",     stra(s1), "abcabXXXXXcabcaXXXXXbc");
   test_int_gt("strreplacea3c",  strsizea(s1), 22);
   test_int("strreplacea3d",     strlena(s1), 22);
   strcopya(s1, "abcab$replcabca$replbc");
   test_int("strreplacea4a",     strreplacea(s1, "$repl", "XXXXXXXX"), 2);
   test_str("strreplacea4b",     stra(s1), "abcabXXXXXXXXcabcaXXXXXXXXbc");
   test_int_gt("strreplacea4c",  strsizea(s1), 28);
   test_int("strreplacea4d",     strlena(s1), 28);
   strcopya(s1, "abcab$replcabca$replbc");
   test_int("strreplacea5a",     strreplacea(s1, "$repl", "XXXXXXXXXXXX"), 2);
   test_str("strreplacea5b",     stra(s1), "abcabXXXXXXXXXXXXcabcaXXXXXXXXXXXXbc");
   test_int_gt("strreplacea5c",  strsizea(s1), 36);
   test_int("strreplacea5d",     strlena(s1), 36);
   strcopya(s1, moogle);
   test_int("strreplacea6",      strreplacea(NULL, "$repl", "XXXXXXXXXXXX"), 0);
   test_int("strreplacea7a",     strreplacea(s1, NULL, "XXXXXXXXXXXX"), 0);
   test_str("strreplacea7b",     stra(s1), moogle);

   /* Test strreplacea_list */
   {
      string_pair repl[] = {  { "$a",  "" },
                              { "$b",  "B" },
                              { "$c",  "CC" },
                              { "$d",  "DDD" },
                              { NULL, NULL }  };
      strcopya(s1, "xxx$ax$bx$cx$dx$axxx");
      test_int("strreplacea_list1",    strreplacea_list(s1, repl), 5);
      test_str("strreplacea_list2",    stra(s1), "xxxxBxCCxDDDxxxx");
      test_int_gt("strreplacea_list3", strsizea(s1), 16);
      test_int("strreplacea_list4",    strlena(s1), 16);
   }

   /* Test sprintfa */
   test_ptr("sprintfa1",      sprintfa(s1, "moogle%s", "/kupo!"), s1);
   test_str("sprintfa2",      stra(s1), "moogle/kupo!");
   test_int_gt("sprintfa3",   strsizea(s1), 12);
   test_int("sprintfa4",      strlena(s1), 12);

   /* Test strdeletea */
   strcopya(s1, "moogle32");
   test_ptr("strdeletea1a",   strdeletea(s1, 32, 4), s1);
   test_str("strdeletea1b",   stra(s1), "moogle32");
   test_int("strdeletea1c",   strlena(s1), 8);
   strcopya(s1, "moogle32");
   test_ptr("strdeletea2a",   strdeletea(s1, 3, 3), s1);
   test_str("strdeletea2b",   stra(s1), "moo32");
   test_int("strdeletea2c",   strlena(s1), 5);
   strcopya(s1, "moogle32");
   test_ptr("strdeletea3a",   strdeletea(s1, 3, 12), s1);
   test_str("strdeletea3b",   stra(s1), "moo");
   test_int("strdeletea3c",   strlena(s1), 3);
   test_ptr("strdeletea4",    strdeletea(NULL, 3, 3), NULL);
   strcopya(s1, "moogle32");
   test_ptr("strtrunca1a",    strtrunca(s1, 5), s1);
   test_str("strtrunca1b",    stra(s1), "moogl");
   test_int("strtrunca1c",    strlena(s1), 5);

   /* Test strinserta */
   strcopya(s1, "moogle32");
   test_ptr("strinserta1a",   strinserta(s1, 32, "moogle"), s1);
   test_str("strinserta1b",   stra(s1), "moogle32moogle");
   test_int("strinserta1c",   strlena(s1), 14);
   strcopya(s1, "moogle32");
   test_ptr("strinserta2a",   strinserta(s1, 6, "moo"), s1);
   test_str("strinserta2b",   stra(s1), "mooglemoo32");
   test_int("strinserta2c",   strlena(s1), 11);
   strcopya(s1, "moogle32");
   test_ptr("strinserta3a",   strinserta(s1, 6, ""), s1);
   test_str("strinserta3b",   stra(s1), "moogle32");
   test_int("strinserta3c",   strlena(s1), 8);
   test_ptr("strinserta4",    strinserta(NULL, 0, "moogle"), NULL);
   strcopya(s1, "moogle32");
   strcopya(s2, "moogle");
   test_ptr("strinsertaa1a",  strinsertaa(s1, 32, s2), s1);
   test_str("strinsertaa1b",  stra(s1), "moogle32moogle");
   test_int("strinsertaa1c",  strlena(s1), 14);
   strcopya(s1, "moogle32");
   strcopya(s2, "moo");
   test_ptr("strinsertaa2a",  strinsertaa(s1, 6, s2), s1);
   test_str("strinsertaa2b",  stra(s1), "mooglemoo32");
   test_int("strinsertaa2c",  strlena(s1), 11);
   strcopya(s1, "moogle32");
   strcopya(s2, NULL);
   test_ptr("strinsertaa3a",  strinsertaa(s1, 6, s2), s1);
   test_str("strinsertaa3b",  stra(s1), "moogle32");
   test_int("strinsertaa3c",  strlena(s1), 8);
   test_ptr("strinsertaa4",   strinsertaa(NULL, 0, s2), NULL);

   /* Summary */
   printf("Total: %d   Success: %d  Fail: %d\n", success + failure, success, failure);
   return(failure > 0 ? 1 : 0);
   strreleasea(&s1);
   strreleasea(&s2);

}
