/* LIBTOKEN                                                 Version 0.1.2
   Copyright (C) 1998   Justin Smith             justins@ugcs.caltech.edu
   ----------------------------------------------------------------------

   This  program  is free software; you can redistribute it and/or modify
   it  under  the terms of the GNU General Public License as published by
   the  Free  Software  Foundation;  either  version 2 of the License, or   
   (at your option) any later version.

   This  program  is  distributed  in  the  hope  that it will be useful,
   but  WITHOUT  ANY  WARRANTY;  without  even  the  implied  warranty of 
   MERCHANTABILITY   or  FITNESS  FOR  A  PARTICULAR  PURPOSE.   See  the  
   GNU General Public License for more details.

   You  should  have  received  a  copy of the GNU General Public License
   along  with  this  program,  in the file COPYING. If not, write to the
   Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA

   ----------------------------------------------------------------------
*/


#ifdef   __LIBTOKEN_SOURCE
#ifndef  __LIBTOKEN__CMD__HH__
#define  __LIBTOKEN__CMD__HH__


/*** Internal Commands - Fundamental Set ***/


int tokenCmdEmpty(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdEcho(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdExec(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdComment(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);


/*** Commands: Variable manipulation ***/


int tokenCmdGet(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdSet(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdUnset(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdGetEnv(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdSetEnv(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdUnsetEnv(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);


/*** Commands: File Inclusion ***/


int tokenCmdInclude(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdIncludeText(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);


/*** Commands: Conditional processing ***/


int tokenCmdIif(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdIf(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdElif(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdElse(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdEndif(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);


/*** Commands: Special state options ***/


int tokenCmdText(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdSquash(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdNewline(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);


/*** Commands: Custom ($$define;) commands ***/


int tokenCmdCustom(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdDefine(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdUndef(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdArg(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);


/*** Commands: runtime debugging ***/


int tokenCmdDumpCmd(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdDumpVar(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdDumpState(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdDumpVersions(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);


/*** Commands: Math Functions ***/


#ifndef NO_MATH_LIB
int tokenCmdAdd(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdSubtract(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdMultiply(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdDivide(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdNegate(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdExp(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdLn(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
#endif // math lib support


/*** Version Commands ***/


int tokenCmdGetVersion(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdSetVersion(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdSupportedVersion(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdGetRealVersion(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);
int tokenCmdVersionID(tokeniser *t, file *fi, file *fo, const char *cmdname, const slinknode *cmdargs, void *param);


#endif   // header include
#endif   // libtoken source